@@ -1,7 +1,7 @@
 /*
  * Alternate global C function prototypes for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
  */
 
 /* command function list */
@@ -22,9 +22,10 @@ char		*get_string();
 char		*parse_equation(), *parse_section(), *parse_var(), *parse_var2(), *parse_expr();
 char		*list_expression(), *list_equation();
 double		gcd(), gcd_verified(), my_round(), multiply_out_unique();
-long		decstrtol();
+long		decstrtol(), max_memory_usage();
 
 void fphandler(int sig);
 void inthandler(int sig);
 void alarmhandler(int sig);
+void exithandler(int sig);
 void resizehandler(int sig);
