@@ -1,5 +1,5 @@
 /*
- * Miscellaneous routines for Mathomatic.
+ * Standard routines for Mathomatic.
  *
  * Copyright (C) 1987-2010 George Gesslein II.
  
@@ -83,7 +83,9 @@ const char	*str;	/* constant string to display */
 #endif
 	printf(_("Please report this bug to the maintainers,\n"));
 	printf(_("along with the entry sequence that caused it.\n"));
+#if	!SILENT
 	printf(_("Type \"help bugs\" for info on how to report bugs found in this program.\n"));
+#endif
 	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
 }
 
@@ -113,7 +115,6 @@ check_err()
 	}
 }
 
-#if	UNIX || CYGWIN
 /*
  * Get the screen (window) width and height from the operating system.
  *
@@ -122,6 +123,7 @@ check_err()
 void
 get_screen_size()
 {
+#if	UNIX || CYGWIN
 	struct winsize	ws;
 
 	ws.ws_col = 0;
@@ -134,15 +136,18 @@ get_screen_size()
 			screen_rows = ws.ws_row;
 		}
 	}
-}
+#else
+	screen_columns = STANDARD_SCREEN_COLUMNS;
+	screen_rows = STANDARD_SCREEN_ROWS;
 #endif
+}
 
 /*
  * Allocate the needed global expression storage arrays.
  * Each is static and can hold n_tokens elements.
  * n_tokens must not change until Mathomatic terminates.
  *
- * init_mem() is only called once upon Mathomatic startup
+ * init_mem() is called only once upon Mathomatic startup
  * before using the symbolic math engine,
  * and can be undone by calling free_mem() below.
  *
@@ -151,6 +156,8 @@ get_screen_size()
 int
 init_mem()
 {
+	if (n_tokens <= 0)
+		return false;
 	if ((scratch = (token_type *) malloc(((n_tokens * 3) / 2) * sizeof(token_type))) == NULL
 	    || (tes = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
 	    || (tlhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
@@ -164,15 +171,15 @@ init_mem()
 	return true;
 }
 
-#if	false
+#if	LIBRARY
 /*
  * Free the global expression storage arrays.
  * After calling this, memory usage is reset and Mathomatic becomes unusable,
  * so do not call unless finished with using the Mathomatic code.
  *
- * This routine is not needed, because when a program exits,
+ * This routine is usually not needed, because when a program exits,
  * all the memory it allocated is released by the operating system.
- * Inclusion of this routine was requested by Tam Hanna.
+ * Inclusion of this routine was requested by Tam Hanna for Symbian OS.
  */
 void
 free_mem()
@@ -326,16 +333,21 @@ clear_all()
 }
 
 /*
- * Return true if the specified equation space is available, allocating if necessary.
+ * Return true if the specified equation space is available,
+ * zeroing and allocating if necessary.
  */
 int
 alloc_espace(i)
 int	i;	/* equation space number */
 {
-	if (i >= N_EQUATIONS)
+	if (i < 0 || i >= N_EQUATIONS)
 		return false;
+	n_lhs[i] = 0;
+	n_rhs[i] = 0;
 	if (lhs[i] && rhs[i])
 		return true;	/* already allocated */
+	if (lhs[i] || rhs[i])
+		return false;	/* something is wrong */
 	lhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
 	if (lhs[i] == NULL)
 		return false;
@@ -348,6 +360,31 @@ int	i;	/* equation space number */
 	return true;
 }
 
+/*
+ * Allocate all equation spaces up to and including an equation space number,
+ * making sure the specified equation number is valid and usable.
+ *
+ * Returns true if successful.
+ */
+int
+alloc_to_espace(en)
+int	en;	/* equation space number */
+{
+	if (en < 0 || en >= N_EQUATIONS)
+		return false;
+	for (;;) {
+		if (en < n_equations)
+			return true;
+		if (n_equations >= N_EQUATIONS)
+			return false;
+		if (!alloc_espace(n_equations)) {
+			warning(_("Memory is exhausted."));
+			return false;
+		}
+		n_equations++;
+	}
+}
+
 /*
  * Allocate or reuse an empty equation space.
  *
@@ -404,13 +441,14 @@ next_espace()
 
 /*
  * Copy equation space "src" to equation space "dest".
+ * "dest" is overwritten.
  */
 void
 copy_espace(src, dest)
 int	src, dest;	/* equation space numbers */
 {
 	if (src == dest) {
-		error("Internal error: copy_espace() source and destination the same.");
+/*		error("Internal error: copy_espace() source and destination the same."); */
 		return;
 	}
 	blt(lhs[dest], lhs[src], n_lhs[src] * sizeof(token_type));
@@ -478,8 +516,10 @@ long	v;	/* standard Mathomatic variable */
 
 /*
  * Substitute every instance of "v" in "equation" with "expression".
+ *
+ * Return true if something was substituted.
  */
-void
+int
 subst_var_with_exp(equation, np, expression, len, v)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to equation side length */
@@ -489,9 +529,10 @@ long		v;		/* variable to substitute with expression */
 {
 	int	j, k;
 	int	level;
+	int	substituted = false;
 
 	if (v == 0 || len <= 0)
-		return;
+		return false;
 	for (j = *np - 1; j >= 0; j--) {
 		if (equation[j].kind == VARIABLE && equation[j].token.variable == v) {
 			level = equation[j].level;
@@ -503,8 +544,15 @@ long		v;		/* variable to substitute with expression */
 			blt(&equation[j], expression, len * sizeof(token_type));
 			for (k = j; k < j + len; k++)
 				equation[k].level += level;
+			substituted = true;
 		}
 	}
+	if (substituted) {
+		if (is_integer_var(v) && !is_integer_expr(expression, len)) {
+			warning(_("Substituting integer variable with non-integer expression."));
+		}
+	}
+	return substituted;
 }
 
 /*
@@ -577,15 +625,17 @@ int		*np;		/* pointer to the returned parsed expression length */
 #if	LIBRARY
 	snprintf(buf, sizeof(buf), "#%+d", pull_number);
 	pull_number++;
-	cp = parse_expr(equation, np, buf);
+	cp = parse_expr(equation, np, buf, true);
+	if (extra_characters(cp))
+		return false;
 	return(cp && *np > 0);
 #else
 	for (;;) {
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		cp = parse_expr(equation, np, cp);
-		if (cp) {
+		cp = parse_expr(equation, np, cp, true);
+		if (cp && !extra_characters(cp)) {
 			break;
 		}
 	}
@@ -683,7 +733,7 @@ int	n;		/* maximum size of "string" in bytes */
 	}
 	input_column = strlen(prompt_str);
 #if	READLINE
-	if (readline_enabled && !echo_input) {
+	if (readline_enabled) {
 		cp = readline(prompt_str);
 		if (cp == NULL) {
 			if (!quiet_mode)
@@ -698,9 +748,7 @@ int	n;		/* maximum size of "string" in bytes */
 			free(cp);
 		}
 	} else {
-		if (!echo_input) {
-			printf("%s", prompt_str);
-		}
+		printf("%s", prompt_str);
 		if (fgets(string, n, stdin) == NULL) {
 			if (!quiet_mode)
 				printf(_("\nEnd of input.\n"));
@@ -708,9 +756,7 @@ int	n;		/* maximum size of "string" in bytes */
 		}
 	}
 #else
-	if (!echo_input) {
-		printf("%s", prompt_str);
-	}
+	printf("%s", prompt_str);
 	if (fgets(string, n, stdin) == NULL) {
 		if (!quiet_mode)
 			printf(_("\nEnd of input.\n"));
@@ -722,7 +768,7 @@ int	n;		/* maximum size of "string" in bytes */
 	if (i >= 0 && string[i] == '\n') {
 		string[i] = '\0';
 	}
-	if ((gfp != stdout && gfp != stderr) || echo_input) {
+	if ((gfp != stdout && gfp != stderr) || (echo_input && !quiet_mode)) {
 		/* Input that is prompted for is now included in the redirected output
 		   of a command to a file, making redirection like logging. */
 		fprintf(gfp, "%s%s\n", prompt_str, string);
@@ -779,6 +825,7 @@ int	en;	/* equation space number the result is in */
 	}
 	free_result_str();
 	result_str = list_equation(en, false);
+	result_en = en;
 	if (gfp == stdout) {
 		return(result_str != NULL);
 	}
@@ -797,6 +844,7 @@ free_result_str()
 		free(result_str);
 		result_str = NULL;
 	}
+	result_en = -1;
 }
 
 /*
@@ -839,7 +887,7 @@ int	*ip, *jp;
 		} else {
 			*ip = cur_equation;
 		}
-		if (*ip < 0 || *ip >= n_equations) {
+		if (*ip < 0 || *ip >= N_EQUATIONS) {
 			error(_("Invalid equation number."));
 			return false;
 		}
@@ -857,7 +905,7 @@ int	*ip, *jp;
 		} else {
 			*jp = cur_equation;
 		}
-		if (*jp < 0 || *jp >= n_equations) {
+		if (*jp < 0 || *jp >= N_EQUATIONS) {
 			error(_("Invalid equation number."));
 			return false;
 		}
@@ -883,7 +931,7 @@ int	*ip, *jp;
  *
  * Returns true if any non-space characters are encountered before the end of the string
  * and an error message is printed.
- * Otherwise just returns false.
+ * Otherwise just returns false indicating everything is OK.
  */
 int
 extra_characters(cp)
@@ -892,6 +940,7 @@ char	*cp;	/* command line string */
 	if (cp) {
 		cp = skip_space(cp);
 		if (*cp) {
+			printf(_("\nError: \"%s\" not required on input line.\n"), cp);
 			error(_("Extra characters or unrecognized argument."));
 			return true;
 		}
@@ -1117,7 +1166,7 @@ int		n1;	/* expression length */
 {
 	int	i;
 
-	for (i = 0; i < n1; i += 2) {
+	for (i = 0; i < n1; i++) {
 		if (p1[i].kind == CONSTANT && isnan(p1[i].token.constant)) {
 			return true;
 		}
@@ -1136,7 +1185,7 @@ int		n1;	/* expression length */
 {
 	int	i;
 
-	for (i = 0; i < n1; i += 2) {
+	for (i = 0; i < n1; i++) {
 		if (p1[i].kind == VARIABLE && (p1[i].token.variable & VAR_MASK) > SIGN) {
 			return false;		/* not numerical (contains a variable) */
 		}
