@@ -34,55 +34,47 @@ static int	last_temp_var = 0;
 #if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /*
  * The plot command.
+ *
+ * All command functions like this return true if successful, or false for failure.
  */
 int
 plot_cmd(cp)
 char	*cp;	/* the command line argument */
 {
-	char	cl[MAX_CMD_LEN];
-	int	free_cp = false;
+	char	cl[MAX_CMD_LEN], *cp1 = NULL;
+	int	free_cp1 = false;
 
 	if (security_level) {
-		error(_("Command disabled."));
+		error(_("Command disabled by security level."));
 		return false;
 	}
-	if (*cp == '\0') {
-		if (current_not_defined()) {
-			return false;
-		}
+	if (n_lhs[cur_equation]) {
 		if (n_rhs[cur_equation]) {
-			cp = list_expression(rhs[cur_equation], n_rhs[cur_equation], 3);
+			cp1 = list_expression(rhs[cur_equation], n_rhs[cur_equation], 3);
 		} else {
-			cp = list_expression(lhs[cur_equation], n_lhs[cur_equation], 3);
-		}
-		if (cp) {
-			free_cp = true;
-		} else {
-			return false;
+			cp1 = list_expression(lhs[cur_equation], n_lhs[cur_equation], 3);
 		}
 	}
-	if (strpbrk(cp, "'\\")) {
-		error(_("Invalid character in expression string."));
-		if (free_cp)
-			free(cp);
-		return false;
-	}
-	if (strchr(cp, 'y')) {
+	if (cp1)
+		free_cp1 = true;
+	else
+		cp1 = "";
+	if (strchr(cp, 'y') || strchr(cp1, 'y')) {
 		printf(_("Performing 3D surface plot...\n"));
-		snprintf(cl, sizeof(cl), "echo 'set grid; splot %s'|gnuplot -persist", cp);
+		snprintf(cl, sizeof(cl), "echo '%s; splot %s %s'|gnuplot -persist", plot_prefix, cp, cp1);
 	} else {
 		printf(_("Performing 2D plot...\n"));
-		snprintf(cl, sizeof(cl), "echo 'set grid; plot %s'|gnuplot -persist", cp);
+		snprintf(cl, sizeof(cl), "echo '%s; plot %s %s'|gnuplot -persist", plot_prefix, cp, cp1);
 	}
-	if (free_cp)
-		free(cp);
+	if (free_cp1)
+		free(cp1);
 	if (shell_out(cl)) {
 		error(_("Error trying to run gnuplot."));
-		printf(_("Command line = \"%s\".\n"), cl);
+		printf(_("Shell command line = \"%s\".\n"), cl);
 		return false;
 #if	!SILENT
 	} else if (debug_level > 0) {
-		printf(_("Command line = \"%s\".\n"), cl);
+		printf(_("Shell command line = \"%s\".\n"), cl);
 #endif
 	}
 	return true;
@@ -98,6 +90,8 @@ char	*cp;	/* the command line argument */
 {
 	int		rv;	/* return value */
 
+	if (extra_characters(cp))	/* Make sure nothing else is on the command line. */
+		return false;
 #if	LIBRARY
 	free_result_str();
 	result_str = strdup(VERSION);
@@ -106,21 +100,36 @@ char	*cp;	/* the command line argument */
 
 	fprintf(gfp, _("\nMathomatic is GNU LGPL version 2.1 licensed software,\n"));
 	fprintf(gfp, _("meaning it is free software that comes with no warranty.\n"));
-	fprintf(gfp, _("Type \"help copyright\" for the copyright and license.\n"));
+	fprintf(gfp, _("Type \"help license\" for the copyright and license.\n"));
 
-	fprintf(gfp, _("\nThe newest version of Mathomatic is always available at\n"));
-	fprintf(gfp, _("the Mathomatic website: http://mathomatic.org\n"));
+	fprintf(gfp, _("\nDocumentation, reference cards, and the newest version of Mathomatic\n"));
+	fprintf(gfp, _("are always available at the Mathomatic website: www.mathomatic.org\n"));
+	fprintf(gfp, _("Feedback, questions, and contributions in truth are always welcomed.\n"));
 	return rv;
 }
 
+/*
+ * Return the maximum amount of memory (in bytes) that this program will use.
+ */
+long
+max_memory_usage()
+{
+	return((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L);
+}
+
 /*
  * Display version info.
  */
 int
 version_report()
 {
+	long	l;
+
 	fprintf(gfp, _("Mathomatic version %s\n"), VERSION);
 	fprintf(gfp, _("Compile-time defines used: "));
+#if	LINUX
+	fprintf(gfp, "LINUX ");
+#endif
 #if	UNIX
 	fprintf(gfp, "UNIX ");
 #endif
@@ -151,8 +160,16 @@ version_report()
 #if	NO_COLOR
 	fprintf(gfp, "NO_COLOR ");
 #endif
-	fprintf(gfp, _("\n\nThe current expression array size is %d tokens,\n"), n_tokens);
-	fprintf(gfp, _("making the maximum memory usage %ld kilobytes.\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
+
+	fprintf(gfp, "\nsizeof(int) = %u bytes, sizeof(long) = %u bytes.\n\n", (unsigned) sizeof(int), (unsigned) sizeof(long));
+
+	fprintf(gfp, _("The current expression array size is %d tokens,\n"), n_tokens);
+	l = max_memory_usage() / 1000L;
+	if (l >= 10000L) {
+		fprintf(gfp, _("making the maximum memory usage approximately %ld megabytes.\n"), l / 1000L);
+	} else {
+		fprintf(gfp, _("making the maximum memory usage approximately %ld kilobytes.\n"), l);
+	}
 #if	SECURE
 	fprintf(gfp, _("Compiled for maximum security.\n"));
 #else
@@ -165,7 +182,7 @@ version_report()
  * The solve command.
  *
  * Return 0 on failure, 1 on total success,
- * or 2 if partial success (solutions didn't totally verify when doing "solve verify").
+ * or 2 if partial success (solved, but solutions didn't verify when doing "solve verify").
  */
 int
 solve_cmd(cp)
@@ -173,12 +190,12 @@ char	*cp;
 {
 	int		i, j;
 	char		buf[MAX_CMD_LEN];
-	long		v;	/* Mathomatic variable */
-	token_type	want;
 	int		diff_sign;
 	int		verify_flag = false, plural_flag = false, once_through, contains_infinity;
-	char		*cp1;
+	char		*cp1, *cp_start;
+	token_type	want;
 
+	cp_start = cp;
 	if (cur_equation < 0 || cur_equation >= n_equations || n_lhs[cur_equation] <= 0 || n_rhs[cur_equation] <= 0) {
 		error(_("No current equation."));
 		return false;
@@ -199,23 +216,34 @@ char	*cp;
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
+		cp_start = cp;
+	}
+	input_column += (cp - cp_start);
+	if ((cp = parse_equation(i, cp)) == NULL) {
+		return false;
 	}
 	if (verify_flag) {
-		cp = parse_var2(&v, cp);
-		if (cp == NULL || extra_characters(cp)) {
-			return false;
+		if (n_lhs[i] != 1 || n_rhs[i] != 0 || lhs[i][0].kind != VARIABLE) {
+			error(_("Can only verify for a single solve variable."));
+			goto fail;
 		}
+		want = lhs[i][0];
 		copy_espace(cur_equation, i);
-		want.level = 1;
-		want.kind = VARIABLE;
-		want.token.variable = v;
 		if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) > 0) {
-			return_result(cur_equation);
-			if (compare_es(cur_equation, i)) {
-				debug_string(0, _("Nothing was done, so no need to verify solution."));
+			if (!return_result(cur_equation)) {
+				goto fail;
+			}
+			if (!solved_equation(cur_equation) || lhs[cur_equation][0].token.variable != want.token.variable) {
+				error(_("Result not a normally solved equation, so cannot verify."));
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
-				return true;
+				return 1;
+			}
+			if (compare_es(cur_equation, i)) {
+				debug_string(0, _("Nothing was done, so no need to verify solutions."));
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+				return 1;
 			}
 			for (j = 0; j < n_rhs[cur_equation]; j += 2) {
 				if (rhs[cur_equation][j].kind == VARIABLE && (rhs[cur_equation][j].token.variable & VAR_MASK) == SIGN) {
@@ -223,33 +251,45 @@ char	*cp;
 					break;
 				}
 			}
-			subst_var_with_exp(lhs[i], &n_lhs[i], rhs[cur_equation], n_rhs[cur_equation], v);
-			subst_var_with_exp(rhs[i], &n_rhs[i], rhs[cur_equation], n_rhs[cur_equation], v);
+			subst_var_with_exp(lhs[i], &n_lhs[i], rhs[cur_equation], n_rhs[cur_equation], want.token.variable);
+			subst_var_with_exp(rhs[i], &n_rhs[i], rhs[cur_equation], n_rhs[cur_equation], want.token.variable);
 			calc_simp(lhs[i], &n_lhs[i]);
 			calc_simp(rhs[i], &n_rhs[i]);
 			once_through = false;
 check_result:
 			contains_infinity = (exp_contains_infinity(lhs[i], n_lhs[i])
 			    || exp_contains_infinity(rhs[i], n_rhs[i]));
-                        if (!contains_infinity && se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
+                        if (se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
 				if (plural_flag)
 					printf(_("Solutions verified.\n"));
 				else
 					printf(_("Solution verified.\n"));
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
-				return true;
+				if (contains_infinity) {
+					error(_("Solution might be incorrect because it contains infinity or NaN."));
+					return 2;
+				} else {
+					return 1;
+				}
 			} else {
 				if (!contains_infinity && !once_through) {
 					symb_flag = symblify;
-					simpa_side(lhs[i], &n_lhs[i], true, false);
-					simpa_side(rhs[i], &n_rhs[i], true, false);
+					simpa_repeat_side(lhs[i], &n_lhs[i], true, false);
+					simpa_repeat_side(rhs[i], &n_rhs[i], true, false);
 					symb_flag = false;
 					once_through = true;
 					goto check_result;
 				}
-				error(_("Solution might be incorrect."));
-#if	true	/* set to false for testing */
+				if (contains_infinity) {
+					error(_("Solution might be incorrect because it contains infinity or NaN."));
+				} else {
+					if (plural_flag)
+						error(_("Solutions might be incorrect."));
+					else
+						error(_("Solution might be incorrect."));
+				}
+#if	1	/* set to #if 0 for testing, #if 1 for normal operation */
 				n_lhs[i] = 0;
 				n_rhs[i] = 0;
 #endif
@@ -261,12 +301,11 @@ check_result:
 #endif
 		}
 	} else {
-		if (parse_equation(i, cp)) {
-			if (solve_espace(i, cur_equation)) {
-				return return_result(cur_equation);
-			}
+		if (solve_espace(i, cur_equation)) {
+			return return_result(cur_equation);
 		}
 	}
+fail:
 	n_lhs[i] = 0;
 	n_rhs[i] = 0;
 	return false;
@@ -303,7 +342,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	int		i;
 	long		v = 0;			/* Mathomatic variable */
 	double		start, end, step = 1.0;
-	int		result_en;
+	int		result_equation;
 	int		n, ns;
 	token_type	*dest, *source;
 	int		count_down;		/* if true, count down, otherwise count up */
@@ -312,15 +351,15 @@ int	product_flag;	/* true for product, otherwise sum */
 	if (current_not_defined()) {
 		return false;
 	}
-	result_en = next_espace();
+	result_equation = next_espace();
 	if (n_rhs[cur_equation]) {
 		ns = n_rhs[cur_equation];
 		source = rhs[cur_equation];
-		dest = rhs[result_en];
+		dest = rhs[result_equation];
 	} else {
 		ns = n_lhs[cur_equation];
 		source = lhs[cur_equation];
-		dest = lhs[result_en];
+		dest = lhs[result_equation];
 	}
 	if (*cp) {
 		cp = parse_var2(&v, cp);
@@ -393,7 +432,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	if (extra_characters(cp))
 		return false;
 	count_down = (end < start);
-	if (fmod(fabs(start - end) / step, 1.0)) {
+	if (fmod(fabs(start - end) / step, 1.0) != 0.0) {
 		warning(_("End value not reached."));
 	}
 	if (product_flag) {
@@ -431,15 +470,16 @@ int	product_flag;	/* true for product, otherwise sum */
 		blt(&dest[n], tlhs, n_tlhs * sizeof(token_type));
 		n += n_tlhs;
 		calc_simp(dest, &n);
+		side_debug(1, dest, n);
 	}
 	if (n_rhs[cur_equation]) {
-		n_rhs[result_en] = n;
-		blt(lhs[result_en], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
-		n_lhs[result_en] = n_lhs[cur_equation];
+		n_rhs[result_equation] = n;
+		blt(lhs[result_equation], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
+		n_lhs[result_equation] = n_lhs[cur_equation];
 	} else {
-		n_lhs[result_en] = n;
+		n_lhs[result_equation] = n;
 	}
-	return return_result(result_en);
+	return return_result(result_equation);
 }
 
 /*
@@ -620,7 +660,7 @@ char	*cp;
 	for (j = i = start; i <= stop; i++) {
 		if (n_lhs[i]) {
 			j = i;
-			simp_sub(i);
+			simp_equation(i);
 		}
 	}
 	stop = j;
@@ -642,9 +682,9 @@ char	*cp;
 			continue;
 		do {
 			flag = false;
-			simp_sub(i);
+			simp_equation(i);
 			for (j = 0; opt_en[j] >= 0; j++) {
-				simp_sub(opt_en[j]);
+				simp_equation(opt_en[j]);
 				if (i != opt_en[j]) {
 					while (find_more(lhs[i], &n_lhs[i], opt_en[j]))
 						flag = true;
@@ -793,21 +833,31 @@ FILE	*ofp;	/* output file pointer */
 	}
 	fprintf(ofp, "case_sensitive\n");
 
-	if (!color_flag) {
-		fprintf(ofp, "no ");
+	if (bold_colors && color_flag) {
+		fprintf(ofp, "bold color\n");
+	} else {
+		if (!color_flag) {
+			fprintf(ofp, "no color\n");
+		} else {
+			fprintf(ofp, "no bold color\n");
+		}
 	}
-	fprintf(ofp, "color\n");
-
-	if (!bold_colors) {
-		fprintf(ofp, "no ");
-	}
-	fprintf(ofp, "bold_colors\n");
 
 	if (!display2d) {
 		fprintf(ofp, "no ");
 	}
 	fprintf(ofp, "display2d\n");
 
+	if (!fractions_display) {
+		fprintf(ofp, "no ");
+	}
+	fprintf(ofp, "fractions_display\n");
+
+	if (quiet_mode) {
+		fprintf(ofp, "no ");
+	}
+	fprintf(ofp, "prompt\n");
+
 	if (!preserve_surds) {
 		fprintf(ofp, "no ");
 	}
@@ -832,10 +882,7 @@ FILE	*ofp;	/* output file pointer */
 	}
 	fprintf(ofp, "right_associative_power\n");
 
-	if (!negate_highest_precedence) {
-		fprintf(ofp, "no ");
-	}
-	fprintf(ofp, "negate_highest_precedence\n");
+	fprintf(ofp, "plot_prefix = %s\n", plot_prefix);
 
 	fprintf(ofp, "special_variable_characters = %s\n", special_variable_characters);
 }
@@ -878,15 +925,17 @@ save_set_options()
 		return false;
 	}
 	if ((fp = fopen(rc_file, "w")) == NULL) {
+		perror(rc_file);
 		error(_("Unable to write to set options startup file."));
 		return false;
 	}
 	fprintf(fp, "; Mathomatic set options loaded at startup.\n");
-	fprintf(fp, "; This file can be edited.\n\n");
+	fprintf(fp, "; This file can be edited or deleted.\n\n");
 	output_options(fp);
 	if (fclose(fp) == 0) {
 		printf(_("All options saved in startup file \"%s\".\n"), rc_file);
 	} else {
+		perror(rc_file);
 		error(_("Error saving set options."));
 		return false;
 	}
@@ -905,8 +954,9 @@ char	*cp;
 {
 	int	i;
 	int	negate;
-	char	*cp1, *option_string;
+	char	*cp1, *option_string, *original_string;
 
+	original_string = cp;
 try_next_param:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
@@ -956,13 +1006,25 @@ try_next_param:
 		}
 		return true;
 	}
+	if (strncasecmp(option_string, "plot_prefix", 4) == 0) {
+		if (negate) {
+			plot_prefix[0] = '\0';
+		} else {
+			my_strlcpy(plot_prefix, cp, sizeof(plot_prefix));
+		}
+		return true;
+	}
 	if (strncasecmp(option_string, "columns", 6) == 0) {
 		if (negate) {
 			screen_columns = 0;
 		} else {
+			if (*cp == '\0') {
+				get_screen_size();
+				goto check_return;
+			}
 			i = decstrtol(cp, &cp1);
 			if (i < 0 || cp == cp1) {
-				error(_("Please specify how wide the screen is; 0 = infinite columns."));
+				error(_("Please specify how wide the screen is; 0 = no column checking."));
 				return false;
 			}
 			cp = cp1;
@@ -971,23 +1033,18 @@ try_next_param:
 		goto check_return;
 	}
 	if (strncasecmp(option_string, "wide", 4) == 0) {
-		if (!negate) {
+		if (negate) {
+			get_screen_size();
+		} else {
 			screen_columns = 0;
 			screen_rows = 0;
-		} else {
-#if	UNIX || CYGWIN
-			get_screen_size();
-#else
-			screen_columns = STANDARD_SCREEN_COLUMNS;
-			screen_rows = STANDARD_SCREEN_ROWS;
-#endif
 		}
 		goto try_next_param;
 	}
 	if (strncasecmp(option_string, "precision", 4) == 0) {
 		i = decstrtol(cp, &cp1);
-		if (i < 0 || i > 14 || cp == cp1) {
-			error(_("Please specify a display precision between 0 and 14 digits."));
+		if (i < 0 || i > 15 || cp == cp1) {
+			error(_("Please specify a display precision between 0 and 15 digits."));
 			return false;
 		}
 		precision = i;
@@ -1017,14 +1074,20 @@ try_next_param:
 		display2d = !negate;
 		goto try_next_param;
 	}
+	if (strncasecmp(option_string, "fraction", 8) == 0) {
+		fractions_display = !negate;
+		goto try_next_param;
+	}
 	if (strncasecmp(option_string, "prompt", 6) == 0) {
 		quiet_mode = negate;
 		goto try_next_param;
 	}
+#if	0
 	if (strncasecmp(option_string, "html", 4) == 0) {
 		html_flag = !negate;
 		goto try_next_param;
 	}
+#endif
 	if (strncasecmp(option_string, "preserve", 8) == 0) {
 		preserve_surds = !negate;
 		goto try_next_param;
@@ -1091,10 +1154,6 @@ try_next_param:
 		right_associative_power = !negate;
 		goto try_next_param;
 	}
-	if (strncasecmp(option_string, "negate_highest_precedence", 6) == 0) {
-		negate_highest_precedence = !negate;
-		goto try_next_param;
-	}
 #if	!SECURE && !LIBRARY
 	if (security_level < 2 && strcmp_tospace(option_string, "save") == 0) {
 		if (rc_file[0] == '\0') {
@@ -1112,6 +1171,7 @@ try_next_param:
 		goto check_return;
 	}
 #endif
+	printf(_("\nCannot process set string \"%s\".\n"), original_string);
 	error(_("Unknown set option."));
 	return false;
 
@@ -1195,27 +1255,33 @@ char	*cp;
 	int	i, j, k;
 	int	i1;
 	char	exists[N_EQUATIONS];
+	char	*cp1;
 
-	if (!get_range_eol(&cp, &i, &j)) {
-		return false;
-	}
+	CLEAR_ARRAY(exists);
 	for (i1 = 0; i1 < N_EQUATIONS; i1++) {
-		exists[i1] = false;
-	}
-	for (i1 = i; i1 <= j; i1++) {
 		if (n_lhs[i1] > 0) {
 			exists[i1] = true;
 		}
 	}
-	for (i1 = i; i1 <= j; i1++) {
-		if (exists[i1]) {
-			k = next_espace();
-			copy_espace(i1, k);
-			if (!return_result(k)) {
-				return false;
+	do {
+		cp1 = cp;
+		if (!get_range(&cp, &i, &j)) {
+			return false;
+		}
+		if (*cp && cp == cp1) {
+			error(_("Invalid equation number range."));
+			return false;
+		}
+		for (i1 = i; i1 <= j; i1++) {
+			if (exists[i1]) {
+				k = next_espace();
+				copy_espace(i1, k);
+				if (!return_result(k)) {
+					return false;
+				}
 			}
 		}
-	}
+	} while (*cp);
 	return true;
 }
 
@@ -1290,8 +1356,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 		}
 	}
 	if (!has_imag || !has_real) {
-		error(_("Failed, expression is not a mix."));
-		return false;
+		warning(_("Expression was not a mix."));
 	}
 	do {
 		simp_loop(dest, &n1);
@@ -1419,9 +1484,10 @@ char	*cp;
 	token_type	*source;
 	int		n;
 	int		diff_sign;
-	char		buf[MAX_CMD_LEN];
+	char		buf[MAX_CMD_LEN], *cp_start;
 	int		factor_flag = false, value_entered;
 
+	cp_start = cp;
 	if (current_not_defined()) {
 		return false;
 	}
@@ -1464,7 +1530,7 @@ char	*cp;
 			return false;
 		}
 		if (iterations == 0) {
-			warning(_("Infinite iterations."));
+			warning(_("Calculation will be iterated until convergence."));
 			iterations = LONG_MAX - 1L;
 		}
 	}
@@ -1495,7 +1561,7 @@ calc_again:
 		}
 		value_entered = true;
 		/* Disguise all variables in the entered expression by making them negative. */
-		/* That way they won't be improperly substituted. */
+		/* That way they won't be improperly substituted in the future. */
 		for (k1 = 0; k1 < n_tlhs; k1 += 2)
 			if (tlhs[k1].kind == VARIABLE)
 				tlhs[k1].token.variable = -tlhs[k1].token.variable;
@@ -1517,6 +1583,7 @@ calc_again:
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		for (l = 0; l < iterations; l++) {
+			side_debug(1, tes, n_tes);
 			blt(trhs, tlhs, n_tlhs * sizeof(token_type));
 			n_trhs = n_tlhs;
 			subst_var_with_exp(trhs, &n_trhs, tes, n_tes, it_v);
@@ -1642,6 +1709,9 @@ char	*cp;
 	return true;
 }
 
+/*
+ * Compare the Right Hand Sides of two equation spaces.
+ */
 static int
 compare_rhs(i, j, diff_signp)
 int	i, j;
@@ -1649,11 +1719,13 @@ int	*diff_signp;
 {
 	int	rv;
 
+/* First, test the compare function by comparing with self: */
 	rv = se_compare(rhs[i], n_rhs[i], rhs[i], n_rhs[i], diff_signp);
 	if (!rv || *diff_signp) {
-		error(_("Error in compare function or too many terms to compare."));
+		error(_("Too many terms to compare."));
 		return false;
 	}
+/* Now do the requested compare: */
 	sign_cmp_flag = true;
 	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], diff_signp);
 	sign_cmp_flag = false;
@@ -1670,27 +1742,27 @@ compare_es(i, j)
 int	i, j;	/* equation space numbers */
 {
 	int	rv;
-	int	diff_sign;
+	int	diff_sign_lhs, diff_sign_rhs;
 
 	if (n_lhs[i] == 0 || n_lhs[j] == 0)
-		return false;
-	/* compare the two left hand sides */
+		return false;	/* empty equation space */
+	if ((n_rhs[i] == 0) != (n_rhs[j] == 0))
+		return false;	/* mix of expression and equation */
+/* Compare the two left hand sides: */
 	sign_cmp_flag = true;
-	rv = se_compare(lhs[i], n_lhs[i], lhs[j], n_lhs[j], &diff_sign);
+	rv = se_compare(lhs[i], n_lhs[i], lhs[j], n_lhs[j], &diff_sign_lhs);
 	sign_cmp_flag = false;
-	if (!rv || diff_sign)
+	if (!rv)
 		return false;
 	if (n_rhs[i] == 0 && n_rhs[j] == 0)
-		return true;	/* two expressions, not equations */
-	if (n_rhs[i] == 0 || n_rhs[j] == 0)
-		return false;	/* mix of expression and equation */
-	/* compare the two right hand sides */
+		return !diff_sign_lhs;	/* two expressions, not equations */
+/* They are equations, so compare the two right hand sides: */
 	sign_cmp_flag = true;
-	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], &diff_sign);
+	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], &diff_sign_rhs);
 	sign_cmp_flag = false;
-	if (!rv || diff_sign)
+	if (!rv)
 		return false;
-	return true;
+	return(diff_sign_lhs == diff_sign_rhs);
 }
 
 /*
@@ -1734,10 +1806,10 @@ char	*cp;
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
-			debug_string(0, _("Simplifying both expressions..."));
+			debug_string(0, _("Completely simplifying both expressions..."));
 			symb_flag = symb;
-			simpa_side(lhs[i], &n_lhs[i], false, true);
-			simpa_side(lhs[j], &n_lhs[j], false, true);
+			simpa_repeat_side(lhs[i], &n_lhs[i], false, true);
+			simpa_repeat_side(lhs[j], &n_lhs[j], false, true);
 			symb_flag = false;
 #if	!SILENT
 			if (debug_level >= 0) {
@@ -1771,10 +1843,10 @@ char	*cp;
 		if (compare_rhs(i, j, &diff_sign)) {
 			goto times_neg1;
 		}
-		debug_string(0, _("Simplifying both equations..."));
+		debug_string(0, _("Completely simplifying both equations..."));
 		symb_flag = symb;
-		simpa_side(rhs[i], &n_rhs[i], false, true);
-		simpa_side(rhs[j], &n_rhs[j], false, true);
+		simpa_repeat_side(rhs[i], &n_rhs[i], false, true);
+		simpa_repeat_side(rhs[j], &n_rhs[j], false, true);
 		symb_flag = false;
 #if	!SILENT
 		if (debug_level >= 0) {
@@ -1802,10 +1874,10 @@ char	*cp;
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
 	}
-	debug_string(0, _("Simplifying both equations..."));
+	debug_string(0, _("Completely simplifying both equations..."));
 	symb_flag = symb;
-	simpa_side(rhs[i], &n_rhs[i], false, false);
-	simpa_side(rhs[j], &n_rhs[j], false, false);
+	simpa_repeat_side(rhs[i], &n_rhs[i], false, false);
+	simpa_repeat_side(rhs[j], &n_rhs[j], false, false);
 	symb_flag = false;
 	if (compare_rhs(i, j, &diff_sign)) {
 		fprintf(gfp, _("Equations are identical.\n"));
@@ -1836,10 +1908,11 @@ times_neg1:
 	}
 	list_proc(lhs[j], n_lhs[j], false);
 	fprintf(gfp, _(") in the second equation.\n"));
-	if (diff_sign)
-		return 0;	/* equations not identical */
-	else
-		return 2;	/* equations identical, but LHS variable name differs */
+#if	LIBRARY
+	return false;
+#else
+	return 2;
+#endif
 }
 
 /*
@@ -1874,7 +1947,7 @@ char	*cp;
 {
 	long		v = 0, v_tmp;		/* Mathomatic variables */
 	int		i, j;
-	int		nleft, nright;
+	int		nleft = 0, nright = 0;
 	double		lcm, d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
 
@@ -1973,7 +2046,7 @@ do_repeat:
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
-		fprintf(gfp, _("Polynomial GCD (%d Euclidean algorithm iteration%s):\n"), j, (j == 1) ? "" : "s");
+		fprintf(gfp, _("Polynomial GCD (%d Euclidean algorithm iterations):\n"), j);
 		fractions_and_group(trhs, &n_trhs);
 		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
@@ -2155,7 +2228,7 @@ long	v;	/* Mathomatic variable */
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
 	subst_var_with_exp(lhs[cur_equation], &n_lhs[cur_equation], rhs[i], n_rhs[i], v);
-	simp_sub(cur_equation);
+	simp_equation(cur_equation);
 	return true;
 }
 
@@ -2202,12 +2275,7 @@ char	*cp;
 #if     LIBRARY
 				free_result_str();
 				result_str = list_equation(i, false);
-				if (result_str == NULL)
-					return false;
-				if (gfp == stdout) {
-	                                displayed = true;
-					continue;
-	                        }
+				result_en = i;
 #endif
 				if (flist_equation(i) > 0) {
 					displayed = true;
@@ -2215,7 +2283,11 @@ char	*cp;
 			}
 		}
 	} while (*cp);
-	return displayed;
+#if	LIBRARY
+	return(result_str != NULL);
+#else
+	return(displayed);
+#endif
 }
 
 /*
@@ -2229,15 +2301,18 @@ char	*cp;
 	char	*cp1;
 	int	export_flag = 0;
 
-	if (strncasecmp(cp, "gnuplot", 4) == 0) {
+	if (strncasecmp(cp, "gnuplot", 3) == 0) {
 		export_flag = 3;
 		cp = skip_param(cp);
-	} else if (strncasecmp(cp, "export", 4) == 0) {
+	} else if (strncasecmp(cp, "export", 3) == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
-	} else if (strncasecmp(cp, "maxima", 4) == 0) {
+	} else if (strncasecmp(cp, "maxima", 3) == 0) {
 		export_flag = 1;
 		cp = skip_param(cp);
+	} else if (strncasecmp(cp, "hexadecimal", 3) == 0) {
+		export_flag = 4;
+		cp = skip_param(cp);
 	}
 	do {
 		cp1 = cp;
@@ -2253,6 +2328,7 @@ char	*cp;
 #if	LIBRARY
 				free_result_str();
 				result_str = list_equation(i, export_flag);
+				result_en = i;
 				if (result_str == NULL)
 					return false;
 				if (gfp == stdout) {
@@ -2357,7 +2433,7 @@ char	*cp;
 	int			i1;
 	long			v1, last_v;		/* Mathomatic variables */
 	int			vc, cnt;		/* variable counts */
-	sort_type		va[MAX_VARS];
+	sort_type		va[MAX_VARS];		/* variable array */
 	token_type		*p1;
 	int			n1;
 	enum language_list	lang_code = 0;		/* default to no programming language */
@@ -2485,7 +2561,7 @@ char	*cp;
 				subst_constants(lhs[i], &n_lhs[i]);
 				subst_constants(rhs[i], &n_rhs[i]);
 				approximate_roots = true;
-				simp_sub(i);
+				simp_equation(i);
 				factorv(lhs[i], &n_lhs[i], IMAGINARY);
 				if (n_rhs[i]) {
 					factorv(rhs[i], &n_rhs[i], IMAGINARY);
@@ -2509,12 +2585,10 @@ char	*cp;
 {
 	int	i, j;
 	int	n;
-	long	last_v, v;	/* Mathomatic variables */
-	char	*cp_start;
-	long	va[MAX_VARS];	/* variable array */
-	int	vc;		/* variable count */
+	long	last_v, v, va[MAX_VARS];	/* Mathomatic variables */
+	int	vc;				/* variable count */
+	char	*cp_start, *cp1;
 	int	found;
-	char	*cp1;
 
 	cp_start = cp;
 	if (current_not_defined()) {
@@ -2525,8 +2599,10 @@ char	*cp;
 		if (strcmp_tospace(cp, "with") == 0) {
 			if (vc)
 				break;
+#if	0	/* allow "with" variable */
 			error(_("No variables specified."));
 			return false;
+#endif
 		}
 		if (vc >= ARR_CNT(va)) {
 			error(_("Too many variables specified."));
@@ -2580,7 +2656,7 @@ char	*cp;
 				}
 				cp1 = skip_param(cp);
 				input_column += (cp1 - cp_start);
-				if ((cp1 = parse_expr(scratch, &n, cp1)) == NULL || n <= 0) {
+				if ((cp1 = parse_expr(scratch, &n, cp1, true)) == NULL || n <= 0) {
 					return false;
 				}
 				goto do_this;
@@ -2592,8 +2668,8 @@ char	*cp;
 			continue;
 		}
 do_this:
-		/* Disguise all variables in the entered expression by making them negative. */
-		/* That way they won't be improperly substituted. */
+		/* Disguise all variables in the entered expression by making them negative; */
+		/* That way they won't be improperly substituted later, allowing variable interchange. */
 		for (j = 0; j < n; j += 2) {
 			if (scratch[j].kind == VARIABLE) {
 				scratch[j].token.variable = -scratch[j].token.variable;
@@ -2613,7 +2689,7 @@ do_this:
 	blt(lhs[i], tlhs, n_tlhs * sizeof(token_type));
 	n_rhs[i] = n_trhs;
 	blt(rhs[i], trhs, n_trhs * sizeof(token_type));
-	simp_sub(i);
+	simp_equation(i);
 	return return_result(i);
 }
 
@@ -2660,7 +2736,7 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i]) {
 			if (quickest_flag) {
-				simp_sub(i);
+				simp_equation(i);
 			} else {
 				simpa_repeat_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
 				simpa_repeat_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
@@ -2743,7 +2819,7 @@ char	*cp;
 				if (k)
 					fprintf(gfp, ":\n");
 				if (quickest_flag) {
-					simp_sub(i1);
+					simp_equation(i1);
 				} else {
 					simpa_repeat_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
 					simpa_repeat_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
@@ -2780,19 +2856,23 @@ char	*cp;
 	int	i1;
 	int	found;
 	long	v;				/* Mathomatic variable */
+	int	valid_range = false, power_flag = false;
+	char	*cp_start;
+#if	!LIBRARY
+	char	*cp1;
 	double	d;
 	char	buf[MAX_CMD_LEN];
-	int	valid_range = false, power_flag = false;
-	char	*cp1;
+#endif
 
+	cp_start = cp;
+#if	!LIBRARY /* no integer factoring in library mode */
 	if (strcmp_tospace(cp, "number") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "numbers") == 0) {
-#if	!LIBRARY
 		repeat_flag = true;
-#endif
 		cp = skip_param(cp);
 	} else {
+#endif
 		if (strcmp_tospace(cp, "power") == 0) {
 			power_flag = true;
 			cp = skip_param(cp);
@@ -2808,28 +2888,43 @@ char	*cp;
 				return false;
 #endif
 		}
+#if	!LIBRARY
 	}
 	if (!valid_range) {
 		do {
 			if (*cp == '\0') {
+retry:
 				my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
 				cp = get_string(buf, sizeof(buf));
 				if (cp == NULL)
 					return false;
+				cp_start = cp;
 			}
 			if (*cp == '\0')
 				return true;
 			for (; *cp; ) {
 				cp1 = cp;
 				d = strtod(cp, &cp);
-				if (cp == cp1) {
-					error(_("Integer expected."));
-					return false;
-				}
 				cp = skip_space(cp);
+				if (*cp && !isdigit(*cp)) {
+					input_column += (cp1 - cp_start);
+					cp = parse_expr(tes, &n_tes, cp1, false);
+					if (cp == NULL)
+						goto retry;
+					cp_start = cp;
+					if (n_tes <= 0)
+						return true;
+					calc_simp(tes, &n_tes);
+					if (n_tes != 1 || tes[0].kind != CONSTANT) {
+						error(_("Integer expected."));
+						goto retry;
+					}
+					d = tes[0].token.constant;
+					cp = skip_space(cp);
+				}
 				if (!factor_one(d)) {
 					error(_("Number too large to factor or not a non-zero integer."));
-					return false;
+					continue;
 				}
 				display_unique();
 				if (is_prime()) {
@@ -2839,6 +2934,7 @@ char	*cp;
 		} while (repeat_flag);
 		return true;
 	}
+#endif
 	if (power_flag) {
 		if (extra_characters(cp))
 			return false;
@@ -3024,17 +3120,24 @@ char	*cp;
 int
 read_cmd(cp)
 char	*cp;
+{
+	if (security_level >= 3) {
+		error(_("Command disabled by security level."));
+		return false;
+	}
+	return read_file(cp);
+}
+
+int
+read_file(cp)
+char	*cp;
 {
 	int	rv;
 	FILE	*fp;
 	char	buf[MAX_CMD_LEN];
 
-	if (security_level >= 3) {
-		error(_("File reading disabled."));
-		return false;
-	}
 	if (*cp == '\0') {
-		error(_("No read file name specified."));
+		error(_("No file name specified."));
 		return false;
 	}
 	snprintf(buf, sizeof(buf), "%s.in", cp);
@@ -3042,7 +3145,8 @@ char	*cp;
 	if (fp == NULL) {
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
-			error(_("Can't open input file."));
+			perror(cp);
+			error(_("Can't open file to read."));
 			return false;
 		}
 	} else {
@@ -3107,9 +3211,9 @@ edit_again:
 #if	CYGWIN
 		cp1 = "notepad";
 #else
-		error("EDITOR environment variable not set.");
-		return false;
+		cp1 = "nano";
 #endif
+		warning("EDITOR environment variable not set; using default editor.");
 	}
 	snprintf(cl, sizeof(cl), "%s %s", cp1, cp);
 	if (shell_out(cl)) {
@@ -3139,7 +3243,7 @@ char	*cp;
 	char	tmp_file[PATH_MAX];
 
 	if (security_level) {
-		error(_("Command disabled."));
+		error(_("Command disabled by security level."));
 		return false;
 	}
 	clean_up();	/* end any redirection */
@@ -3148,6 +3252,7 @@ char	*cp;
 		my_strlcpy(tmp_file, "mathomatic.tmp", sizeof(tmp_file));
 		fp = fopen(tmp_file, "w+");
 		if (fp == NULL) {
+			perror(tmp_file);
 			error(_("Can't create temporary file."));
 			return false;
 		}
@@ -3155,6 +3260,7 @@ char	*cp;
 		my_strlcpy(tmp_file, TMP_FILE, sizeof(tmp_file));
 		fd = mkstemp(tmp_file);
 		if (fd < 0 || (fp = fdopen(fd, "w+")) == NULL) {
+			perror(tmp_file);
 			error(_("Can't create temporary file."));
 			return false;
 		}
@@ -3164,8 +3270,13 @@ char	*cp;
 		list_cmd("all");
 		high_prec = false;
 		gfp = default_out;
-		fclose(fp);
-		rv = edit_sub(tmp_file);
+		if (fclose(fp)) {
+			rv = false;
+			perror(tmp_file);
+			error(_("Writing temporary file failed."));
+		} else {
+			rv = edit_sub(tmp_file);
+		}
 		unlink(tmp_file);
 		return rv;
 	} else {
@@ -3190,12 +3301,12 @@ char	*cp;
 	int	rv;
 
 	if (security_level >= 2) {
-		error(_("Command disabled."));
+		error(_("Command disabled by security level."));
 		return false;
 	}
 	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
-		error(_("No save file name specified."));
+		error(_("No file name specified."));
 		return false;
 	}
 #if	!SILENT
@@ -3213,6 +3324,7 @@ char	*cp;
 #endif
 	fp = fopen(cp, "w");
 	if (fp == NULL) {
+		perror(cp);
 		error(_("Can't create specified save file."));
 		return false;
 	}
@@ -3221,8 +3333,10 @@ char	*cp;
 	rv = list_cmd("all");
 	high_prec = false;
 	gfp = default_out;
-	if (fclose(fp))
+	if (fclose(fp)) {
 		rv = false;
+		perror(cp);
+	}
 	if (rv) {
 #if	!SILENT
 		printf(_("All equations saved in file \"%s\".\n"), cp);
