@@ -1,7 +1,22 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
+ 
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public 
+    License as published by the Free Software Foundation; either 
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of 
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+ 
+    You should have received a copy of the GNU Lesser General Public 
+    License along with this library; if not, write to the Free Software 
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ 
  */
 
 #include "includes.h"
@@ -26,7 +41,7 @@ int
 roots_cmd(cp)
 char	*cp;
 {
-#define	MAX_ROOT	10000.0	/* root limit needed because more roots become more inaccurate and take longer to calculate */
+#define	MAX_ROOT	10000.0	/* Root limit needed because more roots become more inaccurate and take longer to check. */
 
 	complexs	c, c2;
 #if	!SILENT
@@ -36,6 +51,7 @@ char	*cp;
 	double		k, root, radius, theta, radius_root = 0.0;
 	char		buf[MAX_CMD_LEN];
 
+do_repeat:
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter root (positive integer): "), sizeof(prompt_str));
 		if ((cp = get_string(buf, sizeof(buf))) == NULL)
@@ -43,6 +59,7 @@ char	*cp;
 	}
 	root = strtod(cp, &cp);
 	if ((*cp && !isspace(*cp)) || root < 0.0 || root > MAX_ROOT || fmod(root, 1.0) != 0.0) {
+		error(_("Root invalid or out of range."));
 		printf(_("Root must be a positive integer less than or equal to %.0f.\n"), MAX_ROOT);
 		return false;
 	}
@@ -69,7 +86,7 @@ char	*cp;
 		return false;
 	}
 	if (c.re == 0.0 && c.im == 0.0) {
-		return false;
+		return repeat_flag;
 	}
 /* convert to polar coordinates */
 	errno = 0;
@@ -106,15 +123,17 @@ char	*cp;
 			complex_fixup(&check);
 			printf(_("Inverse check:"));
 			if (check.re || check.im == 0.0) {
-				printf(" %.12g", check.re);
+				printf(" %.10g", check.re);
 			}
 			if (check.im) {
-				printf(" %+.12g*i", check.im);
+				printf(" %+.10g*i", check.im);
 			}
 			printf("\n\n");
 #endif
 		}
 	}
+	if (repeat_flag)
+		goto do_repeat;
 	return true;
 }
 
@@ -157,7 +176,7 @@ start_over:
 		i += len + 1;
 		r = complex_pow(c, p);
 
-#if	false
+#if	0
 		printf("(%.14g+%.14gi)^(%.14g+%.14gi) = %.14g+%.14gi\n", c.re, c.im, p.re, p.im, r.re, r.im);
 #endif
 
@@ -190,7 +209,7 @@ start_over:
 		goto start_over;
 	}
 	if (modified) {
-		warning(_("Complex number roots approximated."));
+		debug_string(1, _("Complex number roots approximated."));
 	}
 	return modified;
 }
