@@ -304,23 +304,24 @@ int
 derivative_cmd(cp)
 char	*cp;
 {
-	int		i;
+	int		i, len;
 	long		v = 0;		/* Mathomatic variable */
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
-	int		simplify_flag = true;
+	int		simplify_flag = true, solved;
 
 	if (current_not_defined()) {
 		return false;
 	}
+	solved = solved_equation(cur_equation);
 	if (strcmp_tospace(cp, "nosimplify") == 0) {
 		simplify_flag = false;
 		cp = skip_param(cp);
 	}
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
-		if (!solved_equation(cur_equation)) {
+		if (!solved) {
 			warning(_("Not a solved equation.  Only the RHS will be differentiated."));
 		}
 		source = rhs[cur_equation];
@@ -349,10 +350,12 @@ char	*cp;
 		if (*cp) {
 			order = decstrtol(cp, &cp);
 		}
-		if (*cp || order <= 0) {
+		if (order <= 0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	}
 	if (no_vars(source, *nps, &v)) {
 #if	!SILENT
@@ -381,10 +384,9 @@ char	*cp;
 			printf(_(" %ld times"), order);
 		}
 		if (simplify_flag) {
-			printf(_(" and simplifying...\n"));
-		} else {
-			printf("...\n");
+			printf(_(" and simplifying"));
 		}
+		printf("...\n");
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -401,11 +403,19 @@ char	*cp;
 			elim_loop(dest, &n1);
 		}
 	}
+	*np = n1;
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
+		if (solved && isvarchar('\'')) {
+			len = list_var(lhs[i][0].token.variable, 0);
+			for (l1 = 0; l1 < order && len < (MAX_VAR_LEN - 1); l1++) {
+				var_str[len++] = '\'';
+			}
+			var_str[len] = '\0';
+			parse_var(&lhs[i][0].token.variable, var_str);
+		}
 	}
-	*np = n1;
 	cur_equation = i;
 	return return_result(cur_equation);
 }
@@ -449,10 +459,12 @@ char	*cp;
 		if (*cp) {
 			order = decstrtol(cp, &cp);
 		}
-		if (*cp || order <= 0) {
+		if (order <= 0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	}
 	if (no_vars(source, n, &v)) {
 		error(_("Current expression contains no variables."));
@@ -578,7 +590,8 @@ char	*cp;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if ((cp = parse_expr(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
+		cp = parse_expr(lhs[our], &our_nlhs, cp, true);
+		if (cp == NULL || extra_characters(cp) || our_nlhs <= 0) {
 			return false;
 		}
 	} else {
@@ -782,7 +795,8 @@ char	*cp;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if ((cp = parse_expr(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
+		cp = parse_expr(tes, &n_tes, cp, true);
+		if (cp == NULL || extra_characters(cp) || n_tes <= 0) {
 			return false;
 		}
 	} else {
@@ -815,7 +829,7 @@ char	*cp;
 	}
 
 /* General limit taking, solve for the limit variable: */
-	debug_string(0, "Solving...");
+	debug_string(0, _("Solving..."));
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
