@@ -1,7 +1,22 @@
 /*
  * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
+ 
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public 
+    License as published by the Free Software Foundation; either 
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of 
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+ 
+    You should have received a copy of the GNU Lesser General Public 
+    License along with this library; if not, write to the Free Software 
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ 
  */
 
 #include "includes.h"
@@ -276,7 +291,8 @@ int		factor_flag;
  *
  * If "level_code" is 3, nothing is done.
  *
- * Add 4 to "level_code" to always factor out the GCD of rational coefficients.
+ * Add 4 to "level_code" to always factor out the GCD of rational coefficients
+ * to produce all reduced integer coefficients.
  *
  * Return true if equation side was modified.
  */
@@ -423,16 +439,21 @@ int		level_code;
 					d = equation[i].token.constant;
 				}
 			}
-#if	true
+#if	1
 			if (!gcd_flag && minimum >= 1.0) {
 				minimum = 1.0;
 				break;
 			}
 #endif
-			if ((minimum < 1.0 && fmod(d, 1.0) == 0.0) || (fmod(d, minimum) != 0.0)) {
+			if ((minimum < 1.0) && (fmod(d, 1.0) == 0.0)) {
 				minimum = 1.0;
 				break;
 			}
+/* Make sure division by the number to factor out results in an integer: */
+			if (fmod(d, minimum) != 0.0) {
+				minimum = 1.0;	/* result not an integer */
+				break;
+			}
 			i++;
 			for (; i < *np && equation[i].level > level; i += 2)
 				;
