@@ -1,7 +1,22 @@
 /*
  * General floating point GCD routine and associated code for Mathomatic.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2010 George Gesslein II.
+ 
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public 
+    License as published by the Free Software Foundation; either 
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of 
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+ 
+    You should have received a copy of the GNU Lesser General Public 
+    License along with this library; if not, write to the Free Software 
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ 
  */
 
 #include "includes.h"
@@ -28,7 +43,7 @@ double	d1, d2;
 	}
 	d1 = fabs(d1);
 	d2 = fabs(d2);
-#if	true	/* true for standard gcd(), otherwise returns 0 (failure) if either parameter is 0 */
+#if	1	/* true for standard gcd(), otherwise returns 0 (failure) if either parameter is 0 */
 	if (d1 == 0)
 		return d2;
 	if (d2 == 0)
@@ -147,7 +162,7 @@ double	*denominatorp;	/* returned denominator */
 			return false;
 /* make sure the result is a fully reduced fraction: */
 		divisor = gcd(numerator, denominator);
-		if (divisor > 1.0) {	/* shouldn't happen, but just in case */
+		if (divisor > 1.0) {	/* just in case result isn't fully reduced */
 			numerator /= divisor;
 			denominator /= divisor;
 		}
@@ -155,7 +170,11 @@ double	*denominatorp;	/* returned denominator */
 		if (fabs(k3 - d) > (small_epsilon * fabs(k3))) {
 			return false;	/* result is too inaccurate */
 		}
-/* numerator and denominator are integral */
+		if (fmod(numerator, 1.0) != 0.0 || fmod(denominator, 1.0) != 0.0) {
+			/* Shouldn't happen if everything works. */
+			return false;
+		}
+/* numerator and denominator are guaranteed integral */
 		*numeratorp = numerator;
 		*denominatorp = denominator;
 		return true;
@@ -163,29 +182,6 @@ double	*denominatorp;	/* returned denominator */
 	return false;
 }
 
-/*
- * Display the specified value.
- * If it is equal to a simple fraction, display that too.
- *
- * Return true if a fraction was displayed.
- */
-int
-display_fraction(value)
-double	value;
-{
-	double	d4, d5;
-	int	rv = false;
-
-	f_to_fraction(value, &d4, &d5);
-	fprintf(gfp, "%.*g", precision, value);
-	if (d5 != 1.0) {
-		fprintf(gfp, " = %.*g/%.*g", precision, d4, precision, d5);
-		rv = true;
-	}
-	fprintf(gfp, "\n");
-	return rv;
-}
-
 /*
  * Convert non-integer constants in an equation side to fractions,
  * when exactly equal to simple fractions.
