@@ -54,6 +54,7 @@ int		precision = 14;				/* the display precision for doubles (number of digits)
 int		case_sensitive_flag = true;		/* "set case_sensitive" flag */
 int		factor_int_flag;			/* factor integers when displaying expressions */
 int		display2d = true;			/* "set display2d" flag */
+int		fractions_display = true;		/* "set fraction" flag */
 int		preserve_surds = true;			/* set option to preserve roots like (2^.5) */
 int		rationalize_denominators = true;	/* try to rationalize denominators if true */
 int		modulus_mode = 2;				/* true for mathematically correct modulus */
@@ -63,10 +64,14 @@ int		finance_option;				/* for displaying dollars and cents */
 int		autosolve = true;			/* Allows solving by typing the variable name at the main prompt */
 int		autocalc = true;			/* Allows automatically calculating a numerical expression */
 int		autoselect = true;			/* Allows selecting equation spaces by typing the number */
-char		special_variable_characters[256] = "\\"; /* user defined characters for variable names, 0 terminated */
+#if	LIBRARY
+char		special_variable_characters[256] = "\\"; /* allow backslash in variable names for Latex compatibility */
+#else
+char		special_variable_characters[256] = "'\\"; /* user defined characters for variable names, '\0' terminated */
+#endif
+char		plot_prefix[256] = "set grid; set xlabel \"X\"; set ylabel \"Y\";";	/* prefix fed into gnuplot before the plot command */
 int		integer_coefficients = false;		/* if true, factor out the GCD of rational coefficients */
 int		right_associative_power;		/* if true, evaluate power operators right to left */
-int		negate_highest_precedence = true;	/* if true, negation (-x) has the highest precedence */
 int		power_starstar;				/* if true, display power operator as "**", otherwise "^" */
 #if	!SILENT
 int		debug_level;				/* current debug level */
@@ -114,9 +119,11 @@ sign_array_type	sign_array;		/* for keeping track of unique "sign" variables */
 FILE		*default_out;		/* file pointer where all gfp output goes by default */
 FILE		*gfp;			/* global output file pointer, for dynamically redirecting Mathomatic output */
 jmp_buf		jmp_save;		/* for setjmp(3) to longjmp(3) to when an error happens deep within this code */
+int		eoption;		/* -e option flag */
 int		test_mode;		/* test mode flag (-t) */
+int		demo_mode;		/* demo mode flag (-d), don't load rc file when true */
 int		quiet_mode;		/* quiet mode (-q, don't display prompts) */
-int		echo_input;		/* if true, echo input and don't use readline */
+int		echo_input;		/* if true, echo input */
 int		readline_enabled = true;	/* set to false (-r) to disable readline */
 int		partial_flag;		/* normally true for partial unfactoring, false for "unfactor fraction" */
 int		symb_flag;		/* true during "simplify symbolic", which is not 100% mathematically correct */
@@ -134,6 +141,7 @@ int		security_level;		/* current enforced security level for session  */
 int		repeat_flag;		/* true if the command is to repeat its function */
 
 /* library variables go here */
-char		*result_str;		/* returned result when using as library */
+char		*result_str;		/* returned result text string when using as library */
+int		result_en = -1;		/* equation number of the returned result, if stored in an equation space */
 const char	*error_str;		/* last error string */
 const char	*warning_str;		/* last warning string */
