@@ -22,7 +22,12 @@
  */
 
 #include "includes.h"
-#include "license.h"	/* the current license for Mathomatic */
+#include "license.h"	/* the current software license for Mathomatic */
+
+#define	CMD_REQUIRED_NCHARS	4	/* Only type this many characters to run a Mathomatic command. */
+					/* Set this to a high number like 50 to require all letters of a command to be typed. */
+
+#define	REPEAT_CMD_STR	"repeat"	/* Name of the repeat command.  This command is specially handled and not listed in the table below. */
 
 /*
  * The following structure is used for each Mathomatic command.
@@ -42,14 +47,14 @@ typedef	struct {
  */
 static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
-{	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces." },
+{	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces.", "Related command: calculate" },
 #if	!LIBRARY
 {	"calculate",	NULL,			calculate_cmd,		"[\"factor\"] [variable number-of-iterations]",		"Temporarily plug in values for variables and approximate.", "This command may be preceded with \"repeat\"." },
 #endif
 {	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to quickly restart Mathomatic." },
 {	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations.", "Related commands: simplify, optimize, and variables" },
-{	"compare",	NULL,			compare_cmd,		"[\"symbolic\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if they are mathematically the same." },
-{	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the contents of the specified equation spaces." },
+{	"compare",	NULL,			compare_cmd,		"[\"symbolic\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if mathematically the same." },
+{	"copy",		NULL,			copy_cmd,		"[equation-number-ranges]",				"Duplicate the contents of the specified equation spaces." },
 {	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] [variable or \"all\"] [order]",	"Symbolically differentiate and simplify, order times." },
 {	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-ranges]",			"Display equation spaces in pretty multi-line (2D) fraction format." },
 {	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD.", "This command may be preceded with \"repeat\"." },
@@ -58,30 +63,32 @@ static com_type com_list[] = {
 {	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in." },
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations.", "This command may be preceded with \"repeat\"." },
-{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Show where the slope of the current equation equals zero." },
+{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Show where the slope of the current equation equals zero.", "Helps with finding the minimums and maximums." },
 {	"factor",	"collect",		factor_cmd,		"[\"number\" [integers]] or [\"power\"] [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
-{	"fraction",	"together",		fraction_cmd,		"[equation-number-range]",				"Convert expression to a single simple fraction, if any fractions are in it." },
-{	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help and reference." },
+{	"fraction",	"together",		fraction_cmd,		"[equation-number-range]",				"Convert expression to a single simple fraction." },
+#if	HELP
+{	"help",		NULL,			help_cmd,		"[topics or command-names]",				"Short, built-in help and reference." },
+#endif
 {	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression.", "Related command: real" },
 {	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials order times, then simplify." },
-{	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
-{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental)." },
-{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\" or \"gnuplot\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
-{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule."},
+{	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials.", "This command only works with polynomials." },
+{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental).", "This command is experimental." },
+{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\" or \"gnuplot\" or \"hexadecimal\"] [equation-number-ranges]",	"Display equation spaces in single-line format.", "Options to export to other math programs." },
+{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule.", "This command cannot integrate over singularities." },
 {	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations.", "Related command: code" },
-{	"pause",	NULL,			pause_cmd,		"[text]",						"Wait for user to press the Enter key. Optionally display a message." },
+{	"pause",	NULL,			pause_cmd,		"[text]",						"Display a line of text and wait for user to press the Enter key." },
 #if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
-{	"plot",		NULL,			plot_cmd,		"[expression]",						"Automatically plot expression in 2D or 3D with Gnuplot." },
+{	"plot",		NULL,			plot_cmd,		"[x-range [y-range]] [expressions,]",			"Automatically plot expressions in 2D or 3D with Gnuplot.", "Plots variable x, and y if 3D surface plot." },
 #endif
 {	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum" },
 #if	READLINE
-{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push the specified expressions into the readline history." },
+{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push equation spaces into the readline history for editing.", "Available only if readline is enabled." },
 #endif
 #if	!LIBRARY
 {	"quit",		"exit",			quit_cmd,		"[exit-value]",						"Terminate this program without saving." },
 #endif
 #if	!SECURE
-{	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in." },
+{	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in.", "Related command: save" },
 #endif
 {	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of the current expression.", "Related command: imaginary" },
 {	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
@@ -89,9 +96,9 @@ static com_type com_list[] = {
 #if	!SECURE
 {	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file.", "Related command: read" },
 #endif
-{	"set",		NULL,			set_cmd,		"[[\"no\"] option] ...",				"Display, set, or save current session options.", "Tip: Type \"set\" by itself to show all current option settings." },
-{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick\"] [\"quickest\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions.", "This command may be preceded with \"repeat\"." },
-{	"solve",	NULL,			solve_cmd,		"[\"verify\"] [\"for\"] variable or \"0\"",		"Solve the current equation for a variable or for zero." },
+{	"set",		NULL,			set_cmd,		"[[\"no\"] option] ...",				"Display, set, or save current session options.", "\"set\" by itself will show all current option settings." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick[est]\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions.", "This command may be preceded with \"repeat\"." },
+{	"solve",	NULL,			solve_cmd,		"[\"verify\"] [\"for\"] variable or \"0\"",		"Solve the current equation for a variable or for zero.", "The verify option checks all displayed solutions." },
 {	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product" },
 #if	!LIBRARY
 {	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show total and optionally the average." },
@@ -102,7 +109,7 @@ static com_type com_list[] = {
 {	"version",	NULL,			version_cmd,		"",							"Display Mathomatic version and license information." }
 };
 
-#if	!LIBRARY
+#if	HELP && !LIBRARY
 char	*example_strings[] = {
 	"; The derivative of the absolute value function:\n",
 	"|x|\n",
@@ -115,12 +122,13 @@ char	*example_strings[] = {
 	"; Solve for the other variable and simplify the result:\n",
 	"solve for celsius\n",
 	"simplify\n",
-	"variables ; show all variables that occur in an expression\n",
+	"variables ; show all variables that occur in expressions\n",
 	"pause\n",
 	"\n",
-	"; Expand into a polynomial:\n",
+	"; Expand the following into polynomial form, then refactor:\n",
 	"(x+1)^5\n",
 	"unfactor\n",
+	"simplify\n",
 	NULL
 };
 #endif
@@ -150,8 +158,6 @@ char	*cp;
 		cur_equation = n;
 		return return_result(cur_equation);
 	}
-	n_lhs[n] = 0;
-	n_rhs[n] = 0;
 	return false;
 }
 
@@ -254,8 +260,6 @@ return_ok:
 		cur_equation = n;
 		return return_result(cur_equation);
 	}
-	n_lhs[n] = 0;
-	n_rhs[n] = 0;
 	return false;
 }
 
@@ -303,13 +307,14 @@ char	*cp;
 			i = decstrtol(cp, &cp1) - 1;
 			break;
 		}
-		if (cp == cp1)
+		if (cp == cp1) {
 			return true;	/* treat as comment */
-		cp = cp1;
-		if (i < 0 || i >= n_equations) {
-			error(_("Equation not defined."));
+		}
+		if (!alloc_to_espace(i)) {
+			put_up_arrow(cp - cp_start, _("Equation number out of range."));
 			return false;
 		}
+		cp = cp1;
 		if (*cp == ':') {
 			cp++;
 		}
@@ -338,12 +343,14 @@ char	*cp;
 		return !rv;
 	}
 #endif
+#if	HELP
 /* a quick way to get help */
 	if (*cp == '?') {
 		cp = skip_space(cp + 1);
 		input_column += (cp - cp_start);
 		return(help_cmd(cp));
 	}
+#endif
 /* See if the string pointed to by cp is a command. */
 /* If so, execute it. */
 do_repeat:
@@ -351,14 +358,18 @@ do_repeat:
 	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	len = cp1 - cp;	/* length of possible command name in cp */
-	if (len == 6 && strncasecmp(cp, "repeat", 6) == 0) {
+
+#define COMPARE_COMMAND_NAME(str)	(len >= min(CMD_REQUIRED_NCHARS, strlen(str)) \
+					&& len <= strlen(str) && strncasecmp(cp, str, len) == 0)
+
+	if (COMPARE_COMMAND_NAME(REPEAT_CMD_STR)) {
 		our_repeat_flag = true;
 		cp = skip_space(cp1);
 		goto do_repeat;
 	}
 	for (i = 0; i < ARR_CNT(com_list); i++) {
-		if ((len >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, len) == 0)
-		    || (com_list[i].secondary_name && len >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, len) == 0)) {
+		if (COMPARE_COMMAND_NAME(com_list[i].name)
+		    || (com_list[i].secondary_name && COMPARE_COMMAND_NAME(com_list[i].secondary_name))) {
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
 /* Copy the command line to buf2 and use it, because the original string may be overwritten. */
@@ -390,6 +401,7 @@ do_repeat:
 						fp = fopen(filename, "w");
 					}
 					if (fp == NULL) {
+						perror(filename);
 						error(_("Can't open redirected output file for writing."));
 						return false;
 					}
@@ -422,7 +434,7 @@ do_repeat:
 		}
 	}
 	if (our_repeat_flag) {
-		error(_("Expecting command to repeat."));
+		error(_("Follow \"repeat\" with command to automatically repeat."));
 		return false;
 	}
 /* cp is not a command, so parse the expression */
@@ -432,8 +444,9 @@ do_repeat:
 }
 
 /*
- * Display and process Mathomatic main prompt input that ends with a newline.
- * Used by the read command.
+ * Display and process Mathomatic main prompt input.
+ * Used by the read command and "help examples".
+ * The input string will be shortened by set_error_level().
  *
  * Return true if successful.
  */
@@ -441,21 +454,36 @@ int
 display_process(cp)
 char	*cp;	/* string to process, will be modified */
 {
+	int	len;
+	int	nlt;	/* true if cp is newline terminated */
+
 	if (cp == NULL)
 		return false;
+	len = strlen(cp);
+	if (len > 0)
+		len--;
+	nlt = (cp[len] == '\n');
+	if (!quiet_mode) {
 #if	LIBRARY
-	fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
-#else
-	default_color();
-	input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
-	if (html_flag)
-		printf("<b>%s</b>", cp);	/* make input bold */
-	else
-		printf("%s", cp);
-	if (gfp != stdout && gfp != stderr) {
 		fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
-	}
+#else
+		default_color();
+		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
+		if (html_flag) {
+			input_column -= (strlen(HTML_PROMPT_STR) - strlen(PROMPT_STR));
+			printf("<b>%s</b>", cp);	/* make input bold */
+		} else {
+			printf("%s", cp);
+		}
+		if (!nlt)
+			printf("\n");
+		if (gfp != stdout && gfp != stderr) {
+			fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
+			if (!nlt)
+				fprintf(gfp, "\n");
+		}
 #endif
+	}
 	set_error_level(cp);
 	return process(cp);
 }
@@ -476,12 +504,14 @@ char	*cp;	/* system command string */
 		error_bug("Shelling out disabled.");
 	}
 	reset_attr();
+
 	errno = 0;
 	rv = system(cp);
-	printf("\n");
 	if (rv < 0) {
 		perror("system(3) call failed");
 	}
+
+	printf("\n");
 	default_color();
 	return rv;
 }
@@ -505,6 +535,7 @@ char	*cp;	/* pointer to variable name string */
 	return skip_space(cp);
 }
 
+#if	HELP
 #define P(A)	fprintf(gfp, "%s\n", A)
 
 /*
@@ -542,14 +573,33 @@ char	**cpp;
 
 	for (i = 0; cpp[i]; i++) {
 		cp = strdup(cpp[i]);
-		if (!display_process(cp))
+		if (cp == NULL)
 			return false;
+		if (!display_process(cp)) {
+			free(cp);
+			return false;
+		}
 		free(cp);
 	}
 	return true;
 }
 #endif
 
+/*
+ * Display a row of dashes to underline a title.
+ */
+void
+underline_title(count)
+int	count;
+{
+	int	i;
+
+	for (i = 1; i < count; i++) {
+		fprintf(gfp, "-");
+	}
+	fprintf(gfp, "\n");
+}
+
 /*
  * The help command.
  */
@@ -562,13 +612,37 @@ char	*cp;
 	int	flag;
 	int	row;
 
+	if (*cp == '\0') {
+		/* default help text: */
+		P("Mathomatic is a Computer Algebra System (CAS) and calculator program.");
+		P("To see helpful interactive examples, type \"help examples\".");
+		P("Type \"help equations\" for help with entering expressions and equations.");
+		P("Type \"help all\" for a summary and syntax of all commands.");
+		P("Type \"help usage\" to display the syntax of all commands.");
+		P("Other help keywords: constants, geometry, license, bugs, options.");
+		P("\"help\" or \"?\" followed by a command name will give info on that command.");
+		fprintf(gfp, "These are the %d commands for this version of Mathomatic:\n", ARR_CNT(com_list));
+		for (i = 0; i < ARR_CNT(com_list); i++) {
+			if ((i % 5) == 0)
+				fprintf(gfp, "\n");
+			j = 15 - fprintf(gfp, "%s", com_list[i].name);
+			for (; j > 0; j--)
+				fprintf(gfp, " ");
+		}
+
+		P("\n\nTo select an equation space, type the equation number at the main prompt.");
+		P("To solve the current equation, type the variable name at the main prompt.");
+		return true;
+	}
+
+next_argument:
 	cp1 = cp;
 	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	if (cp1 != cp) {
+		P("*******************************************************************************");
 		/* first, see if the argument matches any command names */
 		flag = false;
-next_argument:
 		for (i = 0; i < ARR_CNT(com_list); i++) {
 			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0
 			    || (com_list[i].secondary_name && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
@@ -576,55 +650,70 @@ next_argument:
 				flag = true;
 			}
 		}
+		if (strncasecmp(cp, "repeat", cp1 - cp) == 0) {
+			P("repeat - Repeat the following command.");
+			P("Usage: repeat command-line");
+			P("Not all commands are repeatable.\n");
+			flag = true;
+		}
 		if (flag) {
-			cp1 = cp = skip_space(cp1);
-			while (*cp1 && !isspace(*cp1))
-				cp1++;
-			if (cp1 != cp) {
-				goto next_argument;
-			}
-			return true;
+			cp = skip_space(cp1);
+			goto next_argument;
+		}
+#if	!LIBRARY
+		if (strncasecmp(cp, "startup", cp1 - cp) == 0) {
+			display_startup_message(gfp);
+			cp = skip_space(cp1);
+			goto next_argument;
+		}
+		if (strncasecmp(cp, "options", cp1 - cp) == 0) {
+			usage(gfp);
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		if (strncasecmp(cp, "examples", cp1 - cp) == 0) {
-#if	LIBRARY
-			error(_("Examples not available."));
-			return false;
-#else
 			return read_examples(example_strings);
-#endif
 		}
+#endif
 		if (strncasecmp(cp, "copyright", cp1 - cp) == 0 || strncasecmp(cp, "license", cp1 - cp) == 0) {
-			P("Copyright and License for Mathomatic");
-			P("------------------------------------\n");
+			underline_title(P("Copyright and License for Mathomatic"));
 
 			fprintf(gfp, "%s", license_string);
-			return true;
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		if (strncasecmp(cp, "bugs", cp1 - cp) == 0) {
-			P("Report bugs on the Launchpad website:");
-			P("http://launchpad.net/mathomatic\n");
+			P("Please report bugs on the Mathomatic project site on Launchpad.net:");
+			P("https://launchpad.net/mathomatic\n");
 
+			P("Launchpad features a complete bug management system.");
 			P("Please include the following information (from the version command):\n");
 
-			return version_report();
+			version_report();
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
-			P("Mathomatic Command Usage Syntax");
-			P("-------------------------------");
-			for (i = 0, row = 3; i < ARR_CNT(com_list); i++) {
+			underline_title(P("Mathomatic Command Usage Syntax"));
+			for (i = 0, row = 3; i < ARR_CNT(com_list);) {
 				fprintf(gfp, "%s %s\n", com_list[i].name, com_list[i].usage);
 				row++;
-				if (gfp == stdout && screen_rows && row >= (screen_rows - 1)) {
-					row = 1;
-					if (!pause_cmd(""))
-						return false;
+				i++;
+				if (i >= ARR_CNT(com_list))
+					break;
+				if (screen_rows && row >= (screen_rows - 3)) {
+					row = 2;
+					if (gfp == stdout) {
+						if (!pause_cmd(""))
+							return false;
+					}
 				}
 			}
-			return true;
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
-			P("Commonly used standard (Euclidean) geometric formulas");
-			P("-----------------------------------------------------");
+			underline_title(P("Commonly used standard (Euclidean) geometric formulas"));
 			P("Triangle of base \"b\" and height \"h\":");
 			P("    area = b*h/2\n");
 
@@ -635,8 +724,7 @@ next_argument:
 			P("    area = d*(a + b)/2\n");
 
 			P("Circle of radius \"r\":");
-			P("    area = pi*r^2                 perimeter = 2*pi*r");
-			P("    area = perimeter*r/2\n");
+			P("    area = pi*r^2                 perimeter = 2*pi*r\n");
 
 			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
 			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h\n");
@@ -646,69 +734,91 @@ next_argument:
 
 			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
 			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
-			return true;
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 
-			P("    - negate (highest precedence)");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
 			P("    * multiply      / divide        % modulus       // integral divide");
 			P("    + add           - subtract");
 			P("    = equate (lowest precedence)\n");
 
+			P("Multiple operators of the same precedence level are grouped left to right.");
+			P("Parentheses are used to override operator precedence and group things together.");
+			P("Valid parentheses characters are () and {}.\n");
+
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
-			P("Predefined constants and variables follow:\n");
+			P("Predefined variables follow:\n");
+
+			P("    sign, sign1, sign2, ... - may only be +1 or -1");
+			P("    integer, integer1, ... - may be any integer value\n");
+
+			P("Absolute value notation \"|x|\" and dual polarity \"+/-x\" are understood.");
+			cp = skip_space(cp1);
+			goto next_argument;
+		}
+		if (strncasecmp(cp, "constants", cp1 - cp) == 0) {
+			P("Constants are double precision floating point values with about 14 decimal");
+			P("digits accuracy.  They can be entered in standard, scientific, or hexadecimal");
+			P("notation.  Excepting named constants, constants always start with a decimal");
+			P("digit (0..9) or a period.\n");
+
+			P("Named constants follow:\n");
 
 			P("    e or e# - the universal constant e (2.7182818284...)");
 			P("    pi or pi# - the universal constant pi (3.1415926535...)");
 			P("    i or i# - the imaginary unit (square root of -1)");
 			P("The above constants may also be used anywhere variables are required.");
-			P("    sign, sign1, sign2, ... - variables that can only be +1 or -1");
-			P("    integer - variable that can only be an integer value");
-			P("    inf - floating point infinity constant (not a variable)\n");
+			P("    inf - floating point infinity constant");
+			P("    nan - invalid floating point result (not enterable)\n");
 
-			P("Absolute value notation \"|x|\" and dual polarity \"+/-x\" are understood.");
-			return true;
+			fprintf(gfp, "The largest valid constant is +/-%.7g (slightly less than 2^1024).\n", DBL_MAX);
+			fprintf(gfp, "The smallest valid constant is +/-%.7g or 0.\n", DBL_MIN);
+			cp = skip_space(cp1);
+			goto next_argument;
+		}
+		if (strncasecmp(cp, "table", cp1 - cp) == 0) {
+			fprintf(gfp, "Command\tUsage\tNotes\n");
+			for (i = 0; i < ARR_CNT(com_list); i++) {
+				fprintf(gfp, "%s", com_list[i].name);
+				fprintf(gfp, "\t%s %s", com_list[i].name, com_list[i].usage);
+				if (com_list[i].secondary_name) {
+					fprintf(gfp, "\tAlternate name for this command: %s", com_list[i].secondary_name);
+				}
+				if (com_list[i].extra) {
+					fprintf(gfp, "\t%s", com_list[i].extra);
+				}
+				fprintf(gfp, "\n");
+			}
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		if (is_all(cp)) {
-			P("Mathomatic Command Summary");
-			P("--------------------------");
-			for (i = 0, row = 3; i < ARR_CNT(com_list); i++) {
+			underline_title(fprintf(gfp, "Mathomatic Version %s Command Summary\n", VERSION));
+			for (i = 0, row = 3; i < ARR_CNT(com_list);) {
 				row += display_command(i);
-				if (gfp == stdout && screen_rows && row >= (screen_rows - 5)) {
+				i++;
+				if (i >= ARR_CNT(com_list))
+					break;
+				if (screen_rows && row >= (screen_rows - 5)) {
 					row = 1;
-					if (!pause_cmd(""))
-						return false;
+					if (gfp == stdout) {
+						if (!pause_cmd(""))
+							return false;
+					}
 				}
 			}
-			fprintf(gfp, "End of list; total of %d different commands.\n", ARR_CNT(com_list));
-			return true;
+			fprintf(gfp, "End of command list.  Total of %d different commands.\n", ARR_CNT(com_list));
+			cp = skip_space(cp1);
+			goto next_argument;
 		}
 		error(_("Unrecognized help topic or command."));
 		return false;
 	}
-	/* default help text: */
-	P("Mathomatic is a Computer Algebra System (CAS) and calculator program.");
-	P("To see example expressions and commands, type \"help examples\".");
-	P("Type \"help equations\" for help with entering expressions and equations.");
-	P("Type \"help all\" for a summary and syntax of all commands.");
-	P("Type \"help usage\" to display the syntax of all commands.");
-	P("Type \"help geometry\" for some commonly used geometric formulas.");
-	P("\"help\" or \"?\" followed by a command name will give info on that command.\n");
-
-	fprintf(gfp, "These are the %d commands:\n", ARR_CNT(com_list));
-	for (i = 0; i < ARR_CNT(com_list); i++) {
-		if ((i % 5) == 0)
-			fprintf(gfp, "\n");
-		j = 15 - fprintf(gfp, "%s", com_list[i].name);
-		for (; j > 0; j--)
-			fprintf(gfp, " ");
-	}
-
-	P("\n\nTo select an equation space, type the equation number at the main prompt.");
-	P("To solve the current equation, type the variable name at the main prompt.");
 	return true;
 }
+#endif
