@@ -244,16 +244,19 @@ integrate_cmd(cp)
 char	*cp;
 {
 	int		i, j;
+	int		len;
 	long		v = 0;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
-	int		definite_flag = false, constant_flag = false;
-	double		d1, integrate_order = 1.0;
+	int		definite_flag = false, constant_flag = false, solved;
+	double		integrate_order = 1.0;
 	char		var_name_buf[MAX_VAR_LEN];
+	long		l1;
 
 	if (current_not_defined()) {
 		return false;
 	}
+	solved = solved_equation(cur_equation);
 	i = next_espace();
 	for (;; cp = skip_param(cp)) {
 		if (strcmp_tospace(cp, "definite") == 0) {
@@ -291,10 +294,12 @@ char	*cp;
 		if (*cp) {
 			integrate_order = strtod(cp, &cp);
 		}
-		if (*cp || integrate_order <= 0 || fmod(integrate_order, 1.0) != 0.0) {
+		if (integrate_order <= 0 || fmod(integrate_order, 1.0) != 0.0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -310,7 +315,7 @@ char	*cp;
 			printf(_("Integrating with respect to (%s)"), var_str);
 		}
 		if (integrate_order != 1.0) {
-			printf(_(" %.0f times"), integrate_order);
+			printf(_(" %.*g times"), precision, integrate_order);
 		}
 		printf(_(" and simplifying...\n"));
 	}
@@ -321,7 +326,7 @@ char	*cp;
 	factorv(source, nps, v);
 	blt(dest, source, *nps * sizeof(token_type));
 	n1 = *nps;
-	for (d1 = 0; d1 < integrate_order; d1++) {
+	for (l1 = 0; l1 < integrate_order; l1++) {
 		if (!int_dispatch(dest, &n1, v, integrate_sub)) {
 			error(_("Integration failed, not a polynomial."));
 			return false;
@@ -381,11 +386,18 @@ char	*cp;
 		n1 += n2;
 	}
 	simpa_side(dest, &n1, false, false);
+	*np = n1;
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
+		if (solved && isvarchar('\'')) {
+			len = list_var(lhs[i][0].token.variable, 0);
+			for (l1 = 0; l1 < integrate_order && len > 0 && var_str[len-1] == '\''; l1++) {
+				var_str[--len] = '\0';
+			}
+			parse_var(&lhs[i][0].token.variable, var_str);
+		}
 	}
-	*np = n1;
 	cur_equation = i;
 	return return_result(cur_equation);
 }
@@ -606,7 +618,7 @@ char	*cp;
 			return false;
 		}
 	}
-#if	true
+#if	1
 	simp_loop(dest, &n1);
 #else
 	simpa_side(dest, &n1, false, false);
@@ -629,16 +641,18 @@ char	*cp;
 {
 	long		v = 0;
 	int		i, j, k, i1, i2;
+	int		len;
 	int		level;
 	int		iterations = 1000;	/* must be even */
 	int		first_size = 0;
-	int		trap_flag, singularity;
+	int		trap_flag, singularity, solved;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
 
 	if (current_not_defined()) {
 		return false;
 	}
+	solved = solved_equation(cur_equation);
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
 		source = rhs[cur_equation];
@@ -663,10 +677,12 @@ char	*cp;
 		if (*cp) {
 			iterations = decstrtol(cp, &cp);
 		}
-		if (*cp || iterations <= 0 || (iterations % 2) != 0) {
+		if (iterations <= 0 || (iterations % 2) != 0) {
 			error(_("Number of partitions must be a positive, even integer."));
 			return false;
 		}
+		if (extra_characters(cp))
+			return false;
 	} else {
 		if (!prompt_var(&v))
 			return false;
@@ -706,7 +722,11 @@ char	*cp;
 		error_huge();
 	}
 #if	!SILENT
-	printf(_("Approximating the definite integral\n"));
+	if (n_rhs[cur_equation]) {
+		printf(_("Approximating the definite integral of the RHS\n"));
+	} else {
+		printf(_("Approximating the definite integral\n"));
+	}
 	if (trap_flag) {
 		printf(_("using the trapezoid method (%d partitions)...\n"), iterations);
 	} else {
@@ -881,11 +901,18 @@ char	*cp;
 #if	!SILENT
 	printf(_("Numerical integration successful:\n"));
 #endif
+	*np = n1;
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
+		if (solved && isvarchar('\'')) {
+			len = list_var(lhs[i][0].token.variable, 0);
+			if (len > 0 && var_str[len-1] == '\'') {
+				var_str[--len] = '\0';
+			}
+			parse_var(&lhs[i][0].token.variable, var_str);
+		}
 	}
-	*np = n1;
 	cur_equation = i;
 	return return_result(cur_equation);
 }
