@@ -1,31 +1,35 @@
 /*
- * This file contains the C functions for the Mathomatic API.
+ * Mathomatic API
+ *
+ * This file contains the required C functions and simple interface
+ * for developers to use the Mathomatic symbolic math library properly.
+ * These functions are included in the library.
  * Refer to this, if you are going to use the Mathomatic code in other projects.
  *
- * Be sure and define "LIBRARY" when compiling the Mathomatic code for this.
- * And be sure to call clear_all() after completing each group of operations,
+ * Be sure to call clear_all() to erase all equation spaces
+ * after completing each group of operations,
  * otherwise the equation spaces will fill up.
- *
- * Copyright (C) 1987-2009 George Gesslein II.
  */
 
 #include "../includes.h"
 #include "mathomatic.h"
 
 /*
- * matho_init() - Initialize Mathomatic.
+ * matho_init() - Initialize the Mathomatic symbolic math library.
  * Call this only once before calling any Mathomatic code.
- * This must be called exactly once upon program startup and not again.
+ * This must be called exactly once upon program startup and not again,
+ * unless free_mem() is called.
  *
  * Returns true if successful.
- * If this returns false, there was not enough memory
+ * If this returns false, there was not enough memory available
  * and Mathomatic cannot be used.
  */
 int
 matho_init()
 {
 	init_gvars();
-	gfp = stdout;
+	default_out = stdout;	/* if default_out is a file that is not stdout, output is logged to that file */
+	gfp = default_out;
 	if (!init_mem()) {
 		return false;
 	}
@@ -35,24 +39,29 @@ matho_init()
 
 /*
  * Process a Mathomatic command or input an expression into an equation space.
- * Input string is in "input", output string is stored in "*outputp".
+ * The input string is in "input", the output string is stored in "*outputp".
  *
  * This function works just like typing something into the Mathomatic prompt.
  *
  * If this returns true (non-zero), the command or input was successful,
- * and the resulting expression output is stored in "*outputp".
+ * and the resulting expression output string is stored in "*outputp".
  * That is a malloc()ed text string which must be free()d after use
  * to return the memory used by the string.
+ * The equation number of the equation space that the output expression
+ * is additionally stored in (if any) is available in the global "result_en",
+ * otherwise result_en = -1.
  *
  * If this returns false, the command or input failed and a text error
- * message is stored in "*outputp".  The error message should NOT be free()d.
+ * message is always stored in "*outputp".
+ * The error message is a constant string and should NOT be free()d.
  *
- * This routine will set "*outputp" to NULL, if there is no resulting expression or message.
- * If there is an error, there will always be an error message string returned at "*outputp",
- * except when there is no input.
+ * Some commands, like the set command, will return no output when successful,
+ * setting "*outputp" to NULL.
+ *
+ * The resulting output string can safely be ignored by calling with "outputp" set to NULL.
  */
 int
-matho_process(char *input, const char **outputp)
+matho_process(char *input, char **outputp)
 {
 	int	i;
 	int	rv;
@@ -60,6 +69,7 @@ matho_process(char *input, const char **outputp)
 	if (outputp)
 		*outputp = NULL;
 	result_str = NULL;
+	result_en = -1;
 	error_str = NULL;
 	warning_str = NULL;
 	if (input == NULL)
@@ -72,7 +82,7 @@ matho_process(char *input, const char **outputp)
 		}
 		if (outputp) {
 			if (error_str) {
-				*outputp = error_str;
+				*outputp = (char *) error_str;
 			} else {
 				*outputp = _("Processing was interrupted.");
 			}
@@ -84,12 +94,18 @@ matho_process(char *input, const char **outputp)
 	set_error_level(input);
 	rv = process(input);
 	if (rv) {
-		if (outputp)
+		if (outputp) {
 			*outputp = result_str;
+		} else {
+			if (result_str) {
+				free(result_str);
+				result_str = NULL;
+			}
+		}
 	} else {
 		if (outputp) {
 			if (error_str) {
-				*outputp = error_str;
+				*outputp = (char *) error_str;
 			} else {
 				*outputp = _("Unknown error.");
 			}
@@ -104,7 +120,11 @@ matho_process(char *input, const char **outputp)
  * Parse an equation or expression and store in the next available equation space,
  * making it the current equation.
  *
- * Input string is in "input", output string is stored in "*outputp".
+ * The input and output strings are expressions, if successful.
+ * The input string is in "input", the output string is stored in "*outputp".
+ * The equation number of the equation space that the output expression
+ * is additionally stored in (if any) is available in the global "result_en",
+ * otherwise result_en = -1.
  *
  * Works the same as matho_process() above, except commands are not allowed,
  * so that variables are not confused with commands.
@@ -112,7 +132,7 @@ matho_process(char *input, const char **outputp)
  * Returns true (non-zero) if successful.
  */
 int
-matho_parse(char *input, const char **outputp)
+matho_parse(char *input, char **outputp)
 {
 	int	i;
 	int	rv;
@@ -120,6 +140,7 @@ matho_parse(char *input, const char **outputp)
 	if (outputp)
 		*outputp = NULL;
 	result_str = NULL;
+	result_en = -1;
 	error_str = NULL;
 	warning_str = NULL;
 	if (input == NULL)
@@ -132,7 +153,7 @@ matho_parse(char *input, const char **outputp)
 		}
 		if (outputp) {
 			if (error_str) {
-				*outputp = error_str;
+				*outputp = (char *) error_str;
 			} else {
 				*outputp = _("Processing was interrupted.");
 			}
@@ -143,18 +164,24 @@ matho_parse(char *input, const char **outputp)
 	}
 	set_error_level(input);
 	i = next_espace();
-#if	true	/* set this true if you want to be able to enter single variable expressions with no solving */
+#if	1	/* set this true if you want to be able to enter single variable expressions with no solving */
 	rv = parse(i, input);
 #else
 	rv = process_parse(i, input);
 #endif
 	if (rv) {
-		if (outputp)
+		if (outputp) {
 			*outputp = result_str;
+		} else {
+			if (result_str) {
+				free(result_str);
+				result_str = NULL;
+			}
+		}
 	} else {
 		if (outputp) {
 			if (error_str) {
-				*outputp = error_str;
+				*outputp = (char *) error_str;
 			} else {
 				*outputp = _("Unknown error.");
 			}
