@@ -25,15 +25,15 @@
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
 #define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
 
-static int	flist();
+static int	flist_sub();
 static int	flist_recurse();
 
-/* ANSI terminal color code array for 8 color ANSI, we don't use black or white */
-/* because the background may be the same color. */
+/* ANSI terminal color code array for 8 color ANSI; we don't use black or white */
+/* because the background may be the same color, so there are only 6 colors here. */
 static int	carray[] = {
-	32,	/* green (default color) */
-	33,	/* yellow */
-	31,	/* red */
+	32,	/* must be green (default color) */
+	33,	/* must be yellow (for warnings) */
+	31,	/* must be red (for errors) */
 	35,	/* magenta */
 	34,	/* blue */
 	36	/* cyan */
@@ -43,9 +43,9 @@ static int	carray[] = {
 /* Used when HTML output and "set color" and "set bold" options are enabled. */
 /* Good looking with a dark background. */
 static char	*bright_html_carray[] = {
-	"#00FF00",	/* bright green (default color) */
-	"#FFFF00",
-	"#FF0000",
+	"#00FF00",	/* must be bright green (default color) */
+	"#FFFF00",	/* must be bright yellow (for warnings) */
+	"#FF0000",	/* must be bright red (for errors) */
 	"#FF9000",
 	"#FF00FF",
 	"#00FFFF",
@@ -53,9 +53,10 @@ static char	*bright_html_carray[] = {
 };
 
 /* Dim HTML color array for color HTML output. */
+/* Used for HTML output with "set color" and "set no bold" options. */
 /* Good looking with a white background. */
 static char	*html_carray[] = {
-	"green",	/* green (default color) */
+	"green",
 	"olive",
 	"red",
 	"maroon",
@@ -64,7 +65,7 @@ static char	*html_carray[] = {
 	"navy"
 };
 
-/* global variables for flist() below */
+/* global variables for flist_equation() below */
 static int	cur_line;	/* current line */
 static int	cur_pos;	/* current position in the current line on the screen */
 
@@ -76,17 +77,16 @@ static int	cur_pos;	/* current position in the current line on the screen */
 void
 reset_attr()
 {
+	fflush(NULL);	/* flush all output */
 	if (color_flag && cur_color >= 0) {
 		if (html_flag) {
 			printf("</font>");
 		} else {
 			printf("\033[0m");
 		}
+		fflush(stdout);	/* flush standard output */
 	}
 	cur_color = -1;
-	if (fflush(NULL)) {	/* flush all output */
-		perror("fflush(3) call failed");
-	}
 }
 
 /*
@@ -127,6 +127,7 @@ void
 default_color()
 {
 	set_color(0);
+	fflush(stdout);
 }
 
 /*
@@ -143,7 +144,8 @@ char	*buf;
 		if (buf[j] == '0')
 			continue;
 		if (buf[j] == '.') {
-			buf[j+2] = '\0';
+			if (buf[j+1])
+				buf[j+2] = '\0';
 		} else {
 			break;
 		}
@@ -152,7 +154,6 @@ char	*buf;
 
 /*
  * Display the expression or equation stored in equation space "n" in single-line format.
- * The equation space is not modified.
  *
  * Return length (number of screen columns) of output line.
  */
@@ -160,13 +161,13 @@ int
 list1_sub(n, export_flag)
 int	n;		/* equation space number */
 int	export_flag;	/* non-zero for exportable format (readable by other math programs) */
-			/* 1 for Maxima, 2 for other, 3 for Gnuplot */
+			/* 1 for Maxima, 2 for other, 3 for Gnuplot, 4 for hexadecimal */
 {
 	int	len = 0;
 
 	if (n_lhs[n] <= 0)
 		return 0;
-	if (!export_flag && !high_prec) {
+	if ((export_flag == 0 || export_flag == 4) && !high_prec) {
 		len += fprintf(gfp, "#%d: ", n + 1);
 	}
 	len += list_proc(lhs[n], n_lhs[n], export_flag);
@@ -231,6 +232,26 @@ int		n2;
 }
 #endif
 
+/*
+ * Return the allocated string name of the given Mathomatic variable,
+ * or NULL if none.
+ *
+ * Does not return the actual variable name, use list_var() for that.
+ */
+char *
+var_name(v)
+long	v;	/* Mathomatic variable */
+{
+	char	*cp = NULL;
+	long	l;
+
+	l = (labs(v) & VAR_MASK) - VAR_OFFSET;
+	if (l >= 0 && l < MAX_VAR_NAMES) {
+		cp = var_names[l];
+	}
+	return cp;
+}
+
 /*
  * Convert the passed Mathomatic variable to an ASCII variable name.
  * The ASCII variable name is stored in the global var_str[].
@@ -240,7 +261,7 @@ int		n2;
  *
  * If (lang_code == 0), use standard Mathomatic format.
  * If (lang_code > 0), make variable compatible with the language defined in the enumeration language_list defined in am.h
- * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other.
+ * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for Gnuplot, -4 for hexadecimal.
  */
 int
 list_var(v, lang_code)
@@ -248,10 +269,11 @@ long	v;		/* variable to convert */
 int	lang_code;	/* language code */
 {
 	int		j;
+	int		from_memory = false;
 	char		*cp = NULL;
 
 	var_str[0] = '\0';
-	switch (v & VAR_MASK) {
+	switch (labs(v) & VAR_MASK) {
 	case V_NULL:
 		return(strlen(var_str));
 	case SIGN:
@@ -269,6 +291,7 @@ int	lang_code;	/* language code */
 			cp = "%i";
 			break;
 		case 0:
+		case -4:
 			cp = "i#";
 			break;
 		case PYTHON:
@@ -288,6 +311,7 @@ int	lang_code;	/* language code */
 			cp = "%e";
 			break;
 		case 0:
+		case -4:
 			cp = "e#";
 			break;
 		case C:
@@ -310,6 +334,7 @@ int	lang_code;	/* language code */
 			cp = "%pi";
 			break;
 		case 0:
+		case -4:
 			cp = "pi#";
 			break;
 		case C:
@@ -330,15 +355,13 @@ int	lang_code;	/* language code */
 		cp = "all";
 		break;
 	default:
-		j = (v & VAR_MASK) - VAR_OFFSET;
-		if (j >= 0) {
-			cp = var_names[j];
-		}
+		cp = var_name(v);
+		from_memory = true;
 		break;
 	}
 	if (cp) {
-		j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
-		if (j) {
+		j = (labs(v) >> VAR_SHIFT) & SUBSCRIPT_MASK;
+		if (j) {	/* for "sign" variables */
 			snprintf(var_str, sizeof(var_str), "%s%d", cp, j - 1);
 		} else {
 			my_strlcpy(var_str, cp, sizeof(var_str));
@@ -346,13 +369,20 @@ int	lang_code;	/* language code */
 	} else {
 		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
+/* Make valid C type variable if exporting or generating code: */
+	if (from_memory && lang_code != 0 && lang_code != -4) {
+		for (j = 0; var_str[j]; j++) {
+			if (var_str[j] != '_' && !isalnum(var_str[j])) {
+				var_str[j] = '_';
+			}
+		}
+	}
 	return(strlen(var_str));
 }
 
 /*
  * Display an expression in single-line format.
  * Use color if color mode is set.
- * The expression is not modified.
  *
  * Return number of characters output (excluding escape sequences).
  */
@@ -361,7 +391,7 @@ list_proc(p1, n, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other, 3 for Gnuplot */
+				/* 1 for Maxima, 2 for other, 3 for Gnuplot, 4 for hexadecimal */
 {
 	return list_string_sub(p1, n, true, NULL, export_flag);
 }
@@ -458,7 +488,8 @@ token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		outflag;	/* if true, output to gfp */
 char		*string;	/* buffer to save output to or NULL pointer */
-int		export_flag;
+int		export_flag;	/* flag for exportable format (usually false) */
+				/* 1 for Maxima, 2 for other, 3 for Gnuplot, 4 for hexadecimal */
 {
 #define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } if (outflag) { fprintf(gfp, "%s", str); } len += strlen(str); }
 
@@ -469,9 +500,9 @@ int		export_flag;
 	int	len = 0;
 	char	buf[500], buf2[500];
 	int	export_precision;
-	int	cflag;		/* color flag */
+	int	cflag, power_flag;
 
-	cflag = (outflag && !export_flag);
+	cflag = (outflag && (export_flag == 0 || export_flag == 4));
 	if (string)
 		string[0] = '\0';
 	if (high_prec)
@@ -480,8 +511,27 @@ int		export_flag;
 		export_precision = DBL_DIG;
 	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
+		power_flag = false;
+#if	1
+		if (export_flag == 0 && !high_prec) {
+			for (j = i - 1; j <= (i + 1); j++) {
+				if ((j - 1) >= 0 && (j + 1) < n
+				    && p1[j].kind == OPERATOR && (p1[j].token.operatr == POWER || p1[j].token.operatr == FACTORIAL)
+				    && p1[j-1].level == p1[j].level
+				    && p1[j+1].level == p1[j].level
+				    && ((j + 2) >= n || p1[j+2].level != (p1[j].level - 1) || (p1[j+2].token.operatr < POWER))
+				    && ((j - 2) < 0 || p1[j-2].level != (p1[j].level - 1) || (p1[j-2].token.operatr < POWER))) {
+					power_flag = true;
+					break;
+				}
+			}
+		}
+#endif
 		j = cur_level - p1[i].level;
-		k = abs(j);
+		if (power_flag)
+			k = abs(j) - 1;
+		else
+			k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
@@ -500,15 +550,18 @@ int		export_flag;
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (export_flag == 3) {
+			if (export_flag == 4) {
+				snprintf(buf, sizeof(buf), "%a", p1[i].token.constant);
+			} else if (export_flag == 3) {
 				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, p1[i].token.constant);
 				trim_zeros(buf);
 			} else if (export_flag || high_prec) {
 				snprintf(buf, sizeof(buf), "%.*g", export_precision, p1[i].token.constant);
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
 			} else {
-				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+				if (p1[i].token.constant < 0.0 && (i + 1) < n && p1[i+1].level == p1[i].level
+				    && (p1[i+1].token.operatr >= POWER)) {
 					snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
 				} else {
 					snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
@@ -587,7 +640,7 @@ int		n;		/* length of expression */
 	for (i = 0; i < n; i++) {
 		switch (p1[i].kind) {
 		case CONSTANT:
-			if (fmod(p1[i].token.constant, 1.0)) {
+			if (fmod(p1[i].token.constant, 1.0) != 0.0) {
 				return false;
 			}
 			break;
@@ -604,7 +657,7 @@ int		n;		/* length of expression */
 }
 
 /*
- * Display an equation as C, Java, or Python code.
+ * Display an equation space as C, Java, or Python code.
  *
  * Return length of output (number of characters).
  */
@@ -637,7 +690,7 @@ int			int_flag;	/* integer arithmetic flag */
 
 /*
  * Output C, Java, or Python code for an expression.
- * Expression might be modified by this function.
+ * Expression might be modified by this function, though it remains equivalent.
  *
  * Return length of output (number of characters).
  */
@@ -708,13 +761,13 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 			if (equation[i].token.constant == 0.0) {
 				equation[i].token.constant = 0.0; /* fix -0 */
 			}
-/* note: here we will need to parenthesize negative numbers to make -2**x work the same with Python: */
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
 				snprintf(buf, sizeof(buf), "%#.*g", DBL_DIG, equation[i].token.constant);
 				trim_zeros(buf);
 			}
+/* Here we will need to parenthesize negative numbers to make -2**x work the same with Python: */
 			if (equation[i].token.constant < 0) {
 				len += fprintf(gfp, "(%s)", buf);
 			} else {
@@ -774,6 +827,9 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
  * Display an equation space in 2D multi-line fraction format.
  * Use color if color mode is set.
  *
+ * The equation sides must first be basically simplified and prepared by fractions_and_group(),
+ * for proper display.
+ *
  * Return the total width of the output (that is, the required number of screen columns)
  * or zero on failure.
  */
@@ -782,12 +838,12 @@ flist_equation(n)
 int	n;	/* equation space number */
 {
 	int	sind;
-	char	buf[30];
+	char	buf[50];
 	int	len, len2, len3, width;
 	int	pos;
-	int	high, low;
-	int	max_line, min_line;
-	int	max2_line, min2_line;
+	int	high = 0, low = 0;
+	int	max_line = 0, min_line = 0;
+	int	max2_line = 0, min2_line = 0;
 
 	if (n_lhs[n] <= 0)
 		return 0;
@@ -795,11 +851,11 @@ int	n;	/* equation space number */
 	cur_line = 0;
 	cur_pos = 0;
 	sind = n_rhs[n];
-	len += flist(lhs[n], n_lhs[n], false, 0, &max_line, &min_line);
+	len += flist_sub(lhs[n], n_lhs[n], false, 0, &max_line, &min_line);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
 make_smaller:
-		len2 = flist(rhs[n], sind, false, 0, &high, &low);
+		len2 = flist_sub(rhs[n], sind, false, 0, &high, &low);
 		if (screen_columns && gfp == stdout && (len + len2) >= screen_columns && sind > 0) {
 			for (sind--; sind > 0; sind--) {
 				if (rhs[n][sind].level == 1 && rhs[n][sind].kind == OPERATOR) {
@@ -817,7 +873,7 @@ make_smaller:
 			max_line = high;
 		if (low < min_line)
 			min_line = low;
-		len3 = flist(&rhs[n][sind], n_rhs[n] - sind, false, 0, &max2_line, &min2_line);
+		len3 = flist_sub(&rhs[n][sind], n_rhs[n] - sind, false, 0, &max2_line, &min2_line);
 	} else {
 		len2 = 0;
 		len3 = 0;
@@ -840,13 +896,13 @@ make_smaller:
 			cur_pos += fprintf(gfp, "%s", buf);
 		}
 		pos += strlen(buf);
-		pos += flist(lhs[n], n_lhs[n], true, pos, &high, &low);
+		pos += flist_sub(lhs[n], n_lhs[n], true, pos, &high, &low);
 		if (n_rhs[n]) {
 			if (cur_line == 0) {
 				cur_pos += fprintf(gfp, "%s", EQUATE_STRING);
 			}
 			pos += strlen(EQUATE_STRING);
-			pos += flist(rhs[n], sind, true, pos, &high, &low);
+			pos += flist_sub(rhs[n], sind, true, pos, &high, &low);
 		}
 		fprintf(gfp, "\n");
 	}
@@ -855,7 +911,7 @@ make_smaller:
 		fprintf(gfp, "\n");
 		for (cur_line = max2_line; cur_line >= min2_line; cur_line--) {
 			cur_pos = 0;
-			flist(&rhs[n][sind], n_rhs[n] - sind, true, 0, &high, &low);
+			flist_sub(&rhs[n][sind], n_rhs[n] - sind, true, 0, &high, &low);
 			fprintf(gfp, "\n");
 		}
 	}
@@ -869,10 +925,12 @@ make_smaller:
  * 0 is the middle line, lines above are positive, lines below are negative.
  * Use color if available.
  *
+ * This function is for internal use only, and not to be called, except by flist_equation().
+ *
  * Return the width of the expression (that is, the required number of screen columns).
  */
 static int
-flist(p1, n, out_flag, pos, highp, lowp)
+flist_sub(p1, n, out_flag, pos, highp, lowp)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		out_flag;
@@ -906,7 +964,7 @@ int		*highp, *lowp;
 	int	len_div;
 	int	level;
 	int	start_level;
-	int	oflag, html_out;
+	int	oflag, html_out, power_flag;
 	int	len = 0, len1, len2;
 	int	high, low;
 	char	buf[500];
@@ -953,12 +1011,29 @@ check_again:
 		}
 	}
 	for (i = ii; i < n; i++) {
+		power_flag = false;
 		if (i == stop_at) {
 			j = cur_level - p1[div_loc].level;
 			k = abs(j) - 1;
 		} else {
+#if	1
+			for (j = i - 1; j <= (i + 1); j++) {
+				if ((j - 1) >= ii && (j + 1) < n
+				    && p1[j].kind == OPERATOR && (p1[j].token.operatr == POWER || p1[j].token.operatr == FACTORIAL)
+				    && p1[j-1].level == p1[j].level
+				    && p1[j+1].level == p1[j].level
+				    && ((j + 2) >= n || p1[j+2].level != (p1[j].level - 1) || (p1[j+2].token.operatr < POWER))
+				    && ((j - 2) < ii || p1[j-2].level != (p1[j].level - 1) || (p1[j-2].token.operatr < POWER))) {
+					power_flag = true;
+					break;
+				}
+			}
+#endif
 			j = cur_level - p1[i].level;
-			k = abs(j);
+			if (power_flag)
+				k = abs(j) - 1;
+			else
+				k = abs(j);
 		}
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
@@ -999,13 +1074,10 @@ check_again:
 			if (low < *lowp)
 				*lowp = low;
 			if (oflag) {
+				/* display fraction bar */
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
-					if (html_out) {
-						fprintf(gfp, "&minus;");
-					} else {
-						fprintf(gfp, "-");
-					}
+					fprintf(gfp, "-");
 				}
 				set_color(cur_level-1);
 			}
@@ -1026,14 +1098,19 @@ check_again:
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (finance_option) {
+			if (p1[i].token.constant == -1.0 && (i == 0 || p1[i-1].level < p1[i].level)
+			    && (i + 1) < n && p1[i].level == p1[i+1].level && p1[i+1].token.operatr == TIMES) {
+				i++;
+				len += snprintf(buf, sizeof(buf), "-");
+			} else if (finance_option) {
 				if (p1[i].token.constant < 0.0) {
-					len += snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+					len += snprintf(buf, sizeof(buf), "(%'.*f)", finance_option, p1[i].token.constant);
 				} else {
-					len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+					len += snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
 				}
 			} else {
-				if (p1[i].token.constant < 0.0 && !negate_highest_precedence) {
+				if (p1[i].token.constant < 0.0 && (i + 1) < n && p1[i+1].level == p1[i].level
+				    && (p1[i+1].token.operatr >= POWER)) {
 					len += snprintf(buf, sizeof(buf), "(%.*g)", precision, p1[i].token.constant);
 				} else {
 					len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
@@ -1083,8 +1160,8 @@ check_again:
 				len += strlen(cp);
 				break;
 			case POWER:
-#if	false
-				if (html_out
+#if	0
+				if (html_out && power_flag
 				    && p1[i-1].level == p1[i].level
 				    && p1[i+1].level == p1[i].level
 				    && p1[i+1].kind == CONSTANT) {
