@@ -21,8 +21,6 @@
 
 #include "includes.h"
 
-#define	INFINITY_NAME	"inf"	/* set this to the name of the infinity constant as displayed by printf(3) */
-
 int	point_flag;		/* point to location of parse error if true */
 
 /*
@@ -41,7 +39,9 @@ char	*cp;
 }
 
 /*
- * Display an up arrow pointing to the error under the input string.
+ * Display an up arrow pointing to the error, if appropriate.
+ * The up arrow is positioned under the input string,
+ * followed by the error message.
  */
 void
 put_up_arrow(cnt, cp)
@@ -51,8 +51,9 @@ char	*cp;	/* error message */
 #if	!SILENT
 	int	 i;
 
-	if (isatty(0) && point_flag && !html_flag) {
-		for (i = 0; i < (input_column + cnt); i++) {
+	cnt += input_column;
+	if (!quiet_mode && point_flag && (screen_columns == 0 || cnt < screen_columns)) {
+		for (i = 0; i < cnt; i++) {
 			printf(" ");
 		}
 		printf("^ ");
@@ -83,11 +84,9 @@ int		i;	/* location of operator to parenthesize in expression */
 	int	j;
 	int	level;
 
-#if	true
 	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
 		error_bug("Internal error in arguments to binary_parenthesize().");
 	}
-#endif
 	level = p1[i].level++;
 	if (p1[i-1].level++ > level) {
 		for (j = i - 2; j >= 0; j--) {
@@ -120,10 +119,8 @@ int		*np;
 		if (equation[i].token.operatr == NEGATE) {
 			/* change negate operator to -1 times the operand: */
 			equation[i].token.operatr = TIMES;
-			if (negate_highest_precedence) {
-				/* make negate top priority by parenthesizing it first: */
-				binary_parenthesize(equation, *np, i);
-			}	/* otherwise negate has same priority as times/divide */
+			/* make negate top priority by parenthesizing it first: */
+			binary_parenthesize(equation, *np, i);
 		}
 	}
 }
@@ -211,7 +208,8 @@ int		allow_space;	/* if false, any space characters terminate parsing */
 		case '(':
 		case '{':
 			if (operand) {
-#if	true
+#if	1
+/* Allow things like (x)(y) and x{y} by defaulting to the times operator.  The result is x*y. */
 				operand = false;
 				equation[n].level = cur_level;
 				equation[n].kind = OPERATOR;
@@ -338,7 +336,13 @@ parse_power:
 			break;
 		case '%':
 			if (operand) {
+#if	1
+/* Allow %e, %i, and %pi by ignoring % in the wrong place. */
+				operand = false;
+				break;
+#else
 				goto syntax_error;
+#endif
 			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
@@ -387,7 +391,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (*cp == '-' && (!negate_highest_precedence || !(isdigit(cp[1]) || cp[1] == '.'))) {
+			if (*cp == '-') {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = -1.0;
 				equation[n].level = cur_level;
@@ -441,7 +445,7 @@ parse_power:
 				}
 			}
 			if (*cp == '(') {
-				put_up_arrow(cp1 - cp_start, _("Named functions currently not implemented."));
+				put_up_arrow(cp1 - cp_start, _("Named functions currently not implemented, except when using m4."));
 				return(NULL);
 			}
 			cp--;
@@ -476,6 +480,8 @@ syntax_error:
 
 /*
  * Parse an equation string into equation space "n".
+ * The result will not necessarily be an equation,
+ * because this can parse any expression or equation.
  *
  * Returns the new string position, or NULL if error.
  * Currently, there can be no more to parse in the string when this returns.
@@ -485,38 +491,43 @@ parse_equation(n, cp)
 int	n;	/* equation space number */
 char	*cp;	/* pointer to the beginning of the equation character string */
 {
+#if	0
 	if (cp == NULL)
 		return NULL;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
-	if ((cp = parse_section(lhs[n], &n_lhs[n], cp, true)) != NULL) {
-		if ((cp = parse_section(rhs[n], &n_rhs[n], cp, true)) != NULL) {
-			if (extra_characters(cp))
-				return NULL;
-			return cp;
+#endif
+	if ((cp = parse_expr(lhs[n], &n_lhs[n], cp, true)) != NULL) {
+		if ((cp = parse_expr(rhs[n], &n_rhs[n], cp, true)) != NULL) {
+			if (!extra_characters(cp))
+				return cp;
 		}
 	}
+	n_lhs[n] = 0;
+	n_rhs[n] = 0;
 	return NULL;
 }
 
 /*
- * Parse an expression with equation space pull if the line starts with "#" followed by an equation number.
+ * Parse an expression (not an equation) string, with equation space pull
+ * if the string starts with "#" followed by an equation number.
  *
  * Returns the new string position, or NULL if error.
- * Currently, there can be no more to parse in the string when this returns.
  */
 char *
-parse_expr(equation, np, cp)
+parse_expr(equation, np, cp, allow_space)
 token_type	*equation;	/* where the parsed expression is stored (equation side) */
 int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
+int		allow_space;
 {
 	int	i;
-	char	*cp1, *cp2;
+	char	*cp1, *cp2, *cp_start;
 
 	if (cp == NULL)
 		return NULL;
+	cp_start = cp;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
@@ -533,12 +544,12 @@ char		*cp;		/* string to parse */
 			i = decstrtol(cp1, &cp2) - 1;
 			break;
 		}
-		if (cp1 == cp2 || *cp2) {
-			error(_("Error parsing equation space number."));
+		if (cp1 == cp2) {
+			put_up_arrow(cp1 - cp_start, _("Error parsing equation space number after #."));
 			return NULL;
 		}
 		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
-			error(_("No expression available in specified equation space."));
+			put_up_arrow(cp1 - cp_start, _("No expression available in specified equation space."));
 			return NULL;
 		}
 		if (n_rhs[i]) {
@@ -552,12 +563,12 @@ char		*cp;		/* string to parse */
 		list_proc(equation, *np, false);
 		fprintf(gfp, "\n");
 #endif
+		while (*cp2 == '=')
+			cp2++;
+		input_column += (cp2 - cp_start);
 		return cp2;
 	}
-	if ((cp = parse_section(equation, np, cp, true)) != NULL) {
-		if (extra_characters(cp))
-			return NULL;
-	}
+	cp = parse_section(equation, np, cp, allow_space);
 	return cp;
 }
 
@@ -582,7 +593,7 @@ char	*cp;
 	int	len;
 	int	(*strcmpfunc)();
 
-	*vp = V_NULL;
+/*	*vp = V_NULL; */
 	if (case_sensitive_flag) {
 		strcmpfunc = strcmp;
 	} else {
@@ -603,12 +614,13 @@ char	*cp;
 		buf[i++] = *cp1++;
 	}
 	buf[i] = '\0';
+
+	if (strcasecmp(buf, NAN_NAME) == 0) {
+		warning(_("Attempt to enter NaN (Not a Number); Converted to variable."));
+	}
 	if (strcasecmp(buf, INFINITY_NAME) == 0) {
 		error(_("Infinity cannot be used as a variable."));
 		return(NULL);
-	} else if (strcasecmp(buf, "nan") == 0) {
-		error(_("NaN (Not a Number) cannot be directly entered."));
-		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
 	} else {
@@ -679,6 +691,7 @@ char	*cp;
 		*vp = vtmp;
 		return cp1;
 	}
+/* for "sign" variables: */
 	if (isdigit(*cp1)) {
 		j = strtol(cp1, &cp1, 10);
 		if (j < 0 || j > MAX_SUBSCRIPT) {
@@ -720,10 +733,23 @@ set_error_level(cp)
 char	*cp;	/* input string */
 {
 	char	*cp1;
+	int	len;
 
 	point_flag = true;
 /* handle comments, newlines, and the DOS EOF character (control-Z) by truncating the string where found */
-	cp1 = strpbrk(cp, ";\n\r\032");
+	cp1 = cp;
+	while ((cp1 = strpbrk(cp1, ";\n\r\032"))) {
+		if (cp1 != cp && *(cp1 - 1) == '\\') {
+			if (*cp1 == ';') {
+/* skip backslash escaped semicolon */
+				point_flag = false;
+				len = strlen(cp1);
+				blt(cp1 - 1, cp1, len + 1);
+				continue;
+			}
+		}
+		break;
+	}
 	if (cp1) {
 		*cp1 = '\0';
 	}
@@ -745,12 +771,12 @@ var_is_const(v, dp)
 long	v;
 double	*dp;
 {
-	switch (v) {
-	case V_E:
+	if (v == V_E) {
 		if (dp)
 			*dp = M_E;
 		return true;
-	case V_PI:
+	}
+	if (v == V_PI) {
 		if (dp)
 			*dp = M_PI;
 		return true;
