@@ -154,9 +154,13 @@ int		allow_divides;	/* allow variable to be right of a divide (negative exponent
 }
 
 /*
- * Factor polynomials by calling pf_sub() for every additive subexpression.
- * Factors repeated factor polynomials (like (x+1)^5)
- * and symbolic factor polynomials (like (x+a)*(x+b)).
+ * Factor polynomials by calling pf_sub() for every additive sub-expression.
+ * Factors repeated factor polynomials (like (x+1)^5) if "do_repeat".
+ * And factors polynomials with symbolic factors (like (x+a)*(x+b)).
+ *
+ * Does not factor polynomials with different numeric factors (like (x+1)*(x+2)).
+ * Because of floating point round-off error,
+ * this routine does not succeed at factoring very large polynomials.
  *
  * Return true if equation side was modified (factored).
  */
@@ -177,21 +181,21 @@ int		do_repeat;
 {
 	int	modified = false;
 	int	i;
-	int	op;
-	int	len;
+	int	count = 0, level_count = 0;
 
-	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-		if (equation[i].level == level) {
-			op = equation[i].token.operatr;
+		switch (equation[i].token.operatr) {
+		case PLUS:
+		case MINUS:
+			count++;
+			if (equation[i].level == level) {
+				level_count++;
+			}
 		}
 	}
-	len = i - loc;
-	switch (op) {
-	case PLUS:
-	case MINUS:
-		modified = pf_sub(equation, np, loc, len, level, do_repeat);
-		break;
+	if (level_count && count > 1) {	/* so we don't factor expressions with only one additive operator */
+		/* try to factor the sub-expression */
+		modified = pf_sub(equation, np, loc, i - loc, level, do_repeat);
 	}
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -227,8 +231,7 @@ int		do_repeat;	/* factor repeated factors flag */
 	int		i, j, k;
 	long		v = 0, v1, last_v;
 	int		len_first = 0;
-	int		loc1;
-	int		loc2, len2 = 0;
+	int		loc1, loc2, len2 = 0;
 	int		loct, lent;
 	int		count;
 	jmp_buf		save_save;
@@ -312,7 +315,7 @@ int		do_repeat;	/* factor repeated factors flag */
 		if (cnt <= 1)
 			goto skip_factor;
 		if (v == 0) {
-#if	true
+#if	1
 			goto skip_factor;
 #else
 /* no polynomial variables found, try differentiating with respect to each variable until one works */
@@ -678,7 +681,7 @@ long		v;
 	n_trhs = llen;
 	blt(tlhs, smaller, slen * sizeof(token_type));
 	n_tlhs = slen;
-#if	true
+#if	1
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
 		blt(jmp_save, save_save, sizeof(jmp_save));
@@ -729,7 +732,34 @@ long		v;
 }
 
 /*
- * Return true if passed expression is entirely integer and any variables
+ * This function returns true if the passed Mathomatic variable
+ * is of type integer.
+ *
+ * Integer variable names start with "integer".
+ */
+int
+is_integer_var(v)
+long	v;
+{
+	char	*cp;
+	int	(*strncmpfunc)();
+
+	if (case_sensitive_flag) {
+		strncmpfunc = strncmp;
+	} else {
+		strncmpfunc = strncasecmp;
+	}
+
+	cp = var_name(v);
+	if (cp && strncmpfunc(cp, V_INTEGER_PREFIX, strlen(V_INTEGER_PREFIX)) == 0)
+		return true;
+	else
+		return false;
+}
+
+/*
+ * This function is a strict test that
+ * returns true if passed expression is entirely integer and all variables
  * are either "integer" or "sign".
  *
  * The result of evaluating the expression must be integer if this returns true.
@@ -741,9 +771,8 @@ token_type	*p1;	/* expression pointer */
 int		n;	/* length of expression */
 {
 	int	i;
-	long	v_integer;
+	long	v;
 
-	parse_var(&v_integer, V_INTEGER_NAME);
 	for (i = 0; i < n; i++) {
 		switch (p1[i].kind) {
 		case OPERATOR:
@@ -751,11 +780,12 @@ int		n;	/* length of expression */
 				return false;
 			break;
 		case CONSTANT:
-			if (fmod(p1[i].token.constant, 1.0))
+			if (fmod(p1[i].token.constant, 1.0) != 0.0)
 				return false;
 			break;
 		case VARIABLE:
-			if (p1[i].token.variable != v_integer && (p1[i].token.variable & VAR_MASK) != SIGN)
+			v = labs(p1[i].token.variable);
+			if (!is_integer_var(v) && (v & VAR_MASK) != SIGN)
 				return false;
 			break;
 		}
@@ -764,7 +794,9 @@ int		n;	/* length of expression */
 }
 
 /*
- * This routine is a modulus (%) simplifier for equation sides.
+ * This routine is the modulus operator (%) simplifier for equation sides.
+ * Using "integer" variables will allow more simplification here.
+ * Globals tlhs[] and trhs[] are wiped out by the polynomial division.
  *
  * Return true if expression was modified.
  */
@@ -802,124 +834,124 @@ int		*np, loc, level;
 	if (modified)	/* make sure the deepest levels are completed, first */
 		return true;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-		if (equation[i].level == level && equation[i].token.operatr == MODULUS) {
-			for (k = i + 2;; k += 2) {
-				if (k >= *np || equation[k].level <= level)
-					break;
+		if (!(equation[i].level == level && equation[i].token.operatr == MODULUS))
+			continue;
+		for (k = i + 2;; k += 2) {
+			if (k >= *np || equation[k].level <= level)
+				break;
+		}
+		len1 = k - (i + 1);
+		last_op2 = 0;
+		for (j = loc; j < *np && equation[j].level >= level; j++) {
+			if (equation[j].level == level && equation[j].kind == OPERATOR) {
+				last_op2 = equation[j].token.operatr;
+				continue;
 			}
-			len1 = k - (i + 1);
-			last_op2 = 0;
-			for (j = loc; j < *np && equation[j].level >= level; j++) {
-				if (equation[j].level == level && equation[j].kind == OPERATOR) {
-					last_op2 = equation[j].token.operatr;
-					continue;
+			if (last_op2 == MODULUS) {
+				continue;
+			}
+			last_op2 = MODULUS;
+			op = 0;
+			for (i1 = k = j + 1; k < *np && equation[k].level > level; k += 2) {
+				if (equation[k].level == (level + 1)) {
+					op = equation[k].token.operatr;
+					i1 = k;
 				}
-				if (last_op2 == MODULUS) {
-					continue;
+			}
+			len2 = k - j;
+			switch (op) {
+			case MODULUS:
+				/* simplify (x%n)%n to x%n */
+				len3 = k - (i1 + 1);
+				if (se_compare(&equation[i+1], len1, &equation[i1+1], len3, &diff_sign)) {
+					blt(&equation[i1], &equation[k], (*np - k) * sizeof(token_type));
+					*np -= len3 + 1;
+					return true;
 				}
-				last_op2 = MODULUS;
-				op = 0;
-				for (i1 = k = j + 1; k < *np && equation[k].level > level; k += 2) {
-					if (equation[k].level == (level + 1)) {
-						op = equation[k].token.operatr;
-						i1 = k;
-					}
-				}
-				len2 = k - j;
-				switch (op) {
-				case MODULUS:
-					/* simplify (x%n)%n to x%n */
-					len3 = k - (i1 + 1);
-					if (se_compare(&equation[i+1], len1, &equation[i1+1], len3, &diff_sign)) {
-						blt(&equation[i1], &equation[k], (*np - k) * sizeof(token_type));
-						*np -= len3 + 1;
-						return true;
-					}
+				break;
+			case TIMES:
+				if (!is_integer_expr(&equation[j], len2))
 					break;
-				case TIMES:
-					if (!is_integer_expr(&equation[j], len2))
+				/* simplify (i%n*j)%n to (i*j)%n if j is integer */
+				for (i2 = i1 = j + 1;; i1 += 2) {
+					if (i1 >= k || equation[i1].level == (level + 1)) {
+						for (; i2 < i1; i2 += 2) {
+							if (equation[i2].level == (level + 2)
+							    && equation[i2].token.operatr == MODULUS) {
+								len3 = i1 - (i2 + 1);
+								if (se_compare(&equation[i+1], len1, &equation[i2+1], len3, &diff_sign)) {
+									blt(&equation[i2], &equation[i1], (*np - i1) * sizeof(token_type));
+									*np -= len3 + 1;
+									return true;
+								}
+							}
+						}
+					}
+					if (i1 >= k)
 						break;
-					/* simplify (i%n*j)%n to (i*j)%n if j is integer */
-					for (i2 = i1 = j + 1;; i1 += 2) {
-						if (i1 >= k || equation[i1].level == (level + 1)) {
-							for (; i2 < i1; i2 += 2) {
-								if (equation[i2].level == (level + 2)
-								    && equation[i2].token.operatr == MODULUS) {
+				}
+				break;
+			case PLUS:
+			case MINUS:
+				/* simplify (i%n+j)%n to (i+j)%n */
+				/* and ((i%n)*j+k)%n to (i*j+k)%n, when j is integer */
+				for (i2 = i1 = j + 1, i3 = j - 1;; i1 += 2) {
+					if (i1 >= k || equation[i1].level == (level + 1)) {
+						for (; i2 < i1; i2 += 2) {
+							if (equation[i2].level == (level + 2)) {
+								switch (equation[i2].token.operatr) {
+								case MODULUS:
 									len3 = i1 - (i2 + 1);
 									if (se_compare(&equation[i+1], len1, &equation[i2+1], len3, &diff_sign)) {
 										blt(&equation[i2], &equation[i1], (*np - i1) * sizeof(token_type));
 										*np -= len3 + 1;
 										return true;
 									}
-								}
-							}
-						}
-						if (i1 >= k)
-							break;
-					}
-					break;
-				case PLUS:
-				case MINUS:
-					/* simplify (i%n+j)%n to (i+j)%n */
-					/* and ((i%n)*j+k)%n to (i*j+k)%n, when j is integer */
-					for (i2 = i1 = j + 1, i3 = j - 1;; i1 += 2) {
-						if (i1 >= k || equation[i1].level == (level + 1)) {
-							for (; i2 < i1; i2 += 2) {
-								if (equation[i2].level == (level + 2)) {
-									switch (equation[i2].token.operatr) {
-									case MODULUS:
-										len3 = i1 - (i2 + 1);
-										if (se_compare(&equation[i+1], len1, &equation[i2+1], len3, &diff_sign)) {
-											blt(&equation[i2], &equation[i1], (*np - i1) * sizeof(token_type));
-											*np -= len3 + 1;
-											return true;
-										}
-										break;
-									case TIMES:
-										i2 = i1 - 2;
-										if (!is_integer_expr(&equation[i3+1], i1 - (i3 + 1))) {
-											break;
-										}
-										for (i4 = i3 + 2; i4 < i1; i4 += 2) {
-											if (equation[i4].level == (level + 3)
-											    && equation[i4].token.operatr == MODULUS) {
-												for (i5 = i4 + 2; i5 < i1 && equation[i5].level > (level + 3); i5 += 2)
-													;
-												len3 = i5 - (i4 + 1);
-												if (se_compare(&equation[i+1], len1, &equation[i4+1], len3, &diff_sign)) {
-													blt(&equation[i4], &equation[i5], (*np - i5) * sizeof(token_type));
-													*np -= len3 + 1;
-													return true;
-												}
-											}
-										}
+									break;
+								case TIMES:
+									i2 = i1 - 2;
+									if (!is_integer_expr(&equation[i3+1], i1 - (i3 + 1))) {
 										break;
 									}
+									for (i4 = i3 + 2; i4 < i1; i4 += 2) {
+										if (equation[i4].level == (level + 3)
+										    && equation[i4].token.operatr == MODULUS) {
+											for (i5 = i4 + 2; i5 < i1 && equation[i5].level > (level + 3); i5 += 2)
+												;
+											len3 = i5 - (i4 + 1);
+											if (se_compare(&equation[i+1], len1, &equation[i4+1], len3, &diff_sign)) {
+												blt(&equation[i4], &equation[i5], (*np - i5) * sizeof(token_type));
+												*np -= len3 + 1;
+												return true;
+											}
+										}
+									}
+									break;
 								}
 							}
-							i3 = i1;
 						}
-						if (i1 >= k)
-							break;
+						i3 = i1;
 					}
-					break;
+					if (i1 >= k)
+						break;
 				}
-				/* Remove integer*n multiples in x for x%n by doing */
-				/* polynomial division x/n and replacing with remainder%n. */
-				/* Globals tlhs[] and trhs[] are wiped out by the polynomial division here. */
-				v = 0;
-				if (poly_div(&equation[j], len2, &equation[i+1], len1, &v)) {
-					uf_pplus(tlhs, &n_tlhs);	/* so integer%(integer^integer) isn't simplified to 0 */
-					if (is_integer_expr(tlhs, n_tlhs)) {
-						if ((*np + (n_trhs - len2)) > n_tokens)
-							error_huge();
-						for (k = 0; k < n_trhs; k++)
-							trhs[k].level += level;
-						blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
-						*np += n_trhs - len2;
-						blt(&equation[j], trhs, n_trhs * sizeof(token_type));
-						return true;
-					}
+				break;
+			}
+			/* Remove integer*n multiples in x for x%n by doing */
+			/* polynomial division x/n and replacing with remainder%n. */
+			/* Globals tlhs[] and trhs[] are wiped out by the polynomial division here. */
+			v = 0;
+			if (poly_div(&equation[j], len2, &equation[i+1], len1, &v)) {
+				uf_pplus(tlhs, &n_tlhs);	/* so integer%(integer^integer) isn't simplified to 0 */
+				if (is_integer_expr(tlhs, n_tlhs)) {
+					if ((*np + (n_trhs - len2)) > n_tokens)
+						error_huge();
+					for (k = 0; k < n_trhs; k++)
+						trhs[k].level += level;
+					blt(&equation[j+n_trhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
+					*np += n_trhs - len2;
+					blt(&equation[j], trhs, n_trhs * sizeof(token_type));
+					return true;
 				}
 			}
 		}
@@ -1681,7 +1713,7 @@ end_div2:
 }
 
 /*
- * Return the size of a subexpression,
+ * Return the size of a sub-expression,
  * minus any constant multiplier.
  */
 int
@@ -1910,6 +1942,7 @@ int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
 	int		rv;
 	int		count = 0;
 
+	CLEAR_ARRAY(last_va);
 	*pp1 = 0.0;
 	*tp1 = -1;
 	rv = *dcodep;
