@@ -3,26 +3,23 @@
  * algorithm that doesn't use much memory by using a windowing sieve buffer.
  *
  * Copyright (C) 2009 George Gesslein II.
+ 
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public 
+    License as published by the Free Software Foundation; either 
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of 
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+ 
+    You should have received a copy of the GNU Lesser General Public 
+    License along with this library; if not, write to the Free Software 
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ 
  */
 
-/*
-Usage: matho-primes [start [stop]] ["twin"] ["pal" [base]]
-
-Generate batches of consecutive prime numbers up to 18 decimal digits.
-If "twin" is specified, output only twin primes.
-If "pal" is specified, output only palindromic primes.
-The palindromic base may be specified, the default is base 10.
-
-or
-
-Usage: matho-primes [options] [start [stop]]
-
-Generate batches of consecutive prime numbers up to 18 decimal digits.
-Options:
-  -t               Output only twin primes.
-  -p base          Output only palindromic primes.
-*/
-
 /*
  * Changes:
  *
@@ -30,6 +27,10 @@ Options:
  * 3/25/06 - made primes buffer variable size.
  * 3/30/08 - Allow long double to be aliased to double when long double isn't supported.
  * 2/11/09 - Cleanup calculation of number of decimal digits and max_integer.
+ * 9/12/10 - General cleanup and added error message for when the requested number of primes to display is not reached.
+ * 10/25/10 - Added -c and -h options.
+ * 10/26/10 - Using usage2() instead of usage() most of the time, now.
+ * 10/28/10 - Fixed to work with 16-bit integers.
  */
 
 #include <stdio.h>
@@ -56,22 +57,25 @@ Options:
 
 #define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
 
-/* Maximum memory usage in bytes; can be set to any size; the larger, the faster: */
+/* Maximum memory usage in bytes; can be set to any size. */
 #define BUFFER_SIZE	2000000
 #if	BUFFER_SIZE >= (INT_MAX / 2) || BUFFER_SIZE <= 0
-#error BUFFER_SIZE too large.
+#warning BUFFER_SIZE out of range, using INT_MAX / 2.
+#undef BUFFER_SIZE
+#define BUFFER_SIZE	min((INT_MAX / 2), 2000000)
 #endif
 
 void		generate_primes(void);
 int		test_pal(long double d, long double base);
-void		usage(void);
-void		usage2(void);
+void		usage(int ev);
+void		usage2(int ev);
 int		get_long_double_int(char *cp, long double *dp);
 long double	powl(), ceill(), sqrtl(), fmodl(), strtold();
 
 long double max_integer;		/* largest value of a long double integral value */
-long double start_value;		/* where to start finding primes */
-long double number;			/* number of primes to display */
+long double start_value = -1.0;		/* where to start finding primes */
+long double number;			/* number of prime lines to display */
+int count_requested = false;		/* true if the number of primes to display is set by "number" above */
 long double default_number = 20;	/* default number of primes to display */
 long double end_value;			/* where to stop finding primes */
 long double skip_multiples[] = {	/* Additive array that skips over multiples of 2, 3, 5, and 7. */
@@ -83,13 +87,12 @@ long double skip_multiples[] = {	/* Additive array that skips over multiples of
 	 2, 6, 4, 2, 4, 2,10, 2
 };	/* sum of all numbers = 210 = (2*3*5*7) */
 
-long double	last_prime = -3.0;
-
 int		pal_flag, twin_flag;
-long double	pal_base = 10;
+long double	pal_base = 10;	/* The palindrome base, if displaying palindromic primes. */
 
-char		*prime;				/* the boolean sieve array for the current window of numbers */
-int		buffer_size;			/* for variable size prime[] buffer */
+char		*prime;		/* The boolean sieve array (buffer) for the current window of numbers being tested for primality; */
+				/* each char (byte) contains true or false, true if prime. */
+int		buffer_size;	/* Number of bytes for variable size prime[] buffer above. */
 
 char		*prog_name = "matho-primes";
 
@@ -102,28 +105,40 @@ main(int argc, char *argv[])
 	int		i;
 	char		buf[1000];
 
-	start_value = -1.0;
+#if	DEBUG
+	fprintf(stderr, "BUFFER_SIZE = %d\n", BUFFER_SIZE);
+#endif
 /* set the highest number this program will work with: */
 	max_integer = powl(10.0L, (long double) (LDBL_DIG));
-	end_value = max_integer;
-	if (end_value == end_value + 1.0) {
+	while (max_integer == max_integer + 1.0) {
 		fprintf(stderr, "Warning: max_integer (%Lg) is too large; size of long double = %u bytes.\n", max_integer, (unsigned) sizeof(long double));
+		max_integer /= 10.0L;
 	}
+	end_value = max_integer;
 	number = 0;
 /* process command line options: */
-	while ((i = getopt(argc, argv, "tp:")) >= 0) {
+	while ((i = getopt(argc, argv, "c:thp:")) >= 0) {
 		switch (i) {
+		case 'c':
+			count_requested = true;
+			if (optarg && !get_long_double_int(optarg, &number)) {
+				usage2(2);
+			}
+			break;
+		case 'h':
+			usage2(0);
+			break;
 		case 't':
 			twin_flag = true;
 			break;
 		case 'p':
 			pal_flag = true;
 			if (optarg && !get_long_double_int(optarg, &pal_base)) {
-				usage2();
+				usage2(2);
 			}
 			break;
 		default:
-			usage2();
+			usage2(2);
 		}
 	}
 	if (argc > optind) {
@@ -136,18 +151,19 @@ main(int argc, char *argv[])
 		if (get_long_double_int(argv[optind], &start_value)) {
 			optind++;
 		} else {
-			usage();
+			usage2(2);
 		}
 		if (argc > optind && isdigit(argv[optind][0])) {
 			if (get_long_double_int(argv[optind], &end_value)) {
 				if (end_value < start_value) {
 					fprintf(stderr, "End value is less than start value.\n");
-					usage();
+					usage2(2);
 				}
 				optind++;
-				number = max_integer;
+				if (number == 0)
+					number = max_integer;
 			} else {
-				usage();
+				usage2(2);
 			}
 		}
 	}
@@ -162,21 +178,28 @@ main(int argc, char *argv[])
 			pal_flag = true;
 			optind++;
 		} else {
-			usage();
+			usage(2);
 		}
 		if (argc > optind) {
 			if (!get_long_double_int(argv[optind], &pal_base)) {
-				usage();
+				usage(2);
 			}
 			optind++;
 		}
 	}
 	if (argc > optind) {
-		usage();
+		if (strncmp(argv[optind], "twin", 4) == 0) {
+			twin_flag = true;
+			optind++;
+		}
+	}
+	if (argc > optind) {
+		fprintf(stderr, "Unrecognized argument.\n");
+		usage(2);
 	}
 	if (pal_base < 2 || pal_base >= INT_MAX) {
-		fprintf(stderr, "Palindromic base must be >= 2.\n");
-		usage();
+		fprintf(stderr, "Palindrome number base must be >= 2.\n");
+		usage(2);
 	}
 	if (start_value < 0.0) {
 get1:
@@ -197,7 +220,11 @@ get1:
 	}
 	if (number == 0) {
 get2:
-		fprintf(stderr, "Enter number of primes to output (%Lg): ", default_number);
+		if (twin_flag) {
+			fprintf(stderr, "Enter number of prime twins to output (%Lg): ", default_number);
+		} else {
+			fprintf(stderr, "Enter number of primes to output (%Lg): ", default_number);
+		}
 		if (fgets(buf, sizeof(buf), stdin) == NULL)
 			exit(1);
 		switch (buf[0]) {
@@ -211,12 +238,16 @@ get2:
 				goto get2;
 			}
 		}
+		count_requested = true;
 	}
 /* allocate the prime[] buffer: */
 	buffer_size = (int) min(BUFFER_SIZE, (end_value - start_value) + 1.0L);
+#if	DEBUG
+	fprintf(stderr, "buffer_size = %d\n", buffer_size);
+#endif
 	prime = (char *) malloc(buffer_size);
 	if (prime == NULL) {
-		fprintf(stderr, "%s: Not enough memory.\n", prog_name);
+		fprintf(stderr, "%s: Not enough memory for buffer_size = %d.\n", prog_name, buffer_size);
 		exit(2);
 	}
 	generate_primes();
@@ -240,11 +271,14 @@ elim_factor(long double arg)
 	if (d >= buffer_size)
 		return;
 	i = (int) d;
+	if (i >= buffer_size)
+		return; 
 	assert(i >= 0);
 	if (arg >= buffer_size) {
 		prime[i] = false;
 	} else {
 		j = (int) arg;
+		assert(j > 0);
 		for (; i < buffer_size; i += j) {
 			prime[i] = false;
 		}
@@ -252,18 +286,18 @@ elim_factor(long double arg)
 }
 
 /*
- * Generate and display "number" consecutive prime numbers,
- * from "start_value" to "end_value".
+ * Generate and display at most "number" consecutive prime numbers,
+ * between "start_value" and "end_value".
  */
 void
 generate_primes(void)
 {
 	int		n, j;
-	long double	count, ii, vv;
+	long double	count, ii, vv, last_prime = -3.0;
 
 	for (count = 0; count < number; start_value += buffer_size) {
 		if (start_value > end_value) {
-			return;
+			goto check_return;
 		}
 		/* generate a batch of consecutive primes with the prime number sieve */
 		memset(prime, true, buffer_size);	/* set the prime array to all true */
@@ -272,21 +306,18 @@ generate_primes(void)
 		elim_factor(3.0L);
 		elim_factor(5.0L);
 		elim_factor(7.0L);
-		ii = 1.0;
-		for (;;) {
+		for (ii = 1.0; ii <= vv;) {
 			for (j = 0; j < ARR_CNT(skip_multiples); j++) {
 				ii += skip_multiples[j];
 				elim_factor(ii);
 			}
-			if (ii > vv)
-				break;
 		}
 		/* display the requested part of the batch of generated prime numbers */
 		for (n = 0; n < buffer_size && count < number; n++) {
-			if (prime[n]) {
+			if (prime[n]) {	/* if prime number */
 				ii = start_value + n;
 				if (ii > end_value) {
-					return;
+					goto check_return;
 				}
 				if (ii > 1.0) {
 					if (pal_flag && !test_pal(ii, pal_base))
@@ -294,7 +325,7 @@ generate_primes(void)
 					if (twin_flag) {
 						if ((last_prime + 2.0L) == ii) {
 							printf("%.0Lf %.0Lf\n", last_prime, ii);
-							count += 2;
+							count++;
 						}
 					} else {
 						printf("%.0Lf\n", ii);
@@ -305,6 +336,11 @@ generate_primes(void)
 			}
 		}
 	}
+check_return:
+	if (count_requested && count < number) {
+		fprintf(stderr, "%s: Number of primes requested not reached.\n", prog_name);
+		exit(1);
+	}
 }
 
 /*
@@ -352,7 +388,7 @@ get_long_double_int(char *cp, long double *dp)
 }
 
 /*
- * Return true if "d" is a palindrome base "base".
+ * Return true if "d" is a palindromic number, base "base".
  */
 int
 test_pal(long double d, long double base)
@@ -365,7 +401,7 @@ test_pal(long double d, long double base)
 	/* build the array of digits[] */
 	for (i = 0; d >= 1.0; i++) {
 		assert(i < MAX_DIGITS);
-		digits[i] = (int) fmodl(d, base);
+		digits[i] = (int) (fmodl(d, base));
 		d /= base;
 	}
 	/* compare the array of digits[] end to end */
@@ -377,25 +413,27 @@ test_pal(long double d, long double base)
 }
 
 void
-usage(void)
+usage(int ev)
 {
-	printf("\nPrime number generator version 1.0\n");
+	printf("\nPrime number generator version 1.1\n");
 	printf("Usage: %s [start [stop]] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
 	printf("Generate consecutive prime numbers from start to stop up to %Lg.\n", max_integer);
 	printf("If \"twin\" is specified, output only twin primes.\n");
 	printf("If \"pal\" is specified, output only palindromic primes.\n");
-	printf("The palindromic base may be specified, the default is base 10.\n");
-	exit(2);
+	printf("The palindrome number base may be specified, the default is base 10.\n");
+	exit(ev);
 }
 
 void
-usage2(void)
+usage2(int ev)
 {
-	printf("\nPrime number generator version 1.0\n");
+	printf("\nPrime number generator version 1.1\n");
 	printf("Usage: %s [options] [start [stop]]\n\n", prog_name);
 	printf("Generate consecutive prime numbers from start to stop up to %Lg.\n", max_integer);
 	printf("Options:\n");
-	printf("  -t               Output only twin primes.\n");
+	printf("  -c count         Count lines of primes, stop when count reached.\n");
+	printf("  -h               Display this help and exit.\n");
 	printf("  -p base          Output only palindromic primes.\n");
-	exit(2);
+	printf("  -t               Output only twin primes.\n");
+	exit(ev);
 }
