@@ -16,13 +16,14 @@ void set_sign_array(void);
 int next_sign(long *vp);
 void clear_all(void);
 int alloc_espace(int i);
+int alloc_to_espace(int en);
 int alloc_next_espace(void);
 int next_espace(void);
 void copy_espace(int src, int dest);
 int solved_equation(int i);
 int found_var(token_type *p1, int n, long v);
 int var_in_equation(int i, long v);
-void subst_var_with_exp(token_type *equation, int *np, token_type *expression, int len, long v);
+int subst_var_with_exp(token_type *equation, int *np, token_type *expression, int len, long v);
 int min_level(token_type *expression, int n);
 int get_default_en(char *cp);
 int get_expr(token_type *equation, int *np);
@@ -53,6 +54,7 @@ int check_divide_by_zero(double denominator);
 /* cmds.c */
 int plot_cmd(char *cp);
 int version_cmd(char *cp);
+long max_memory_usage(void);
 int version_report(void);
 int solve_cmd(char *cp);
 int sum_cmd(char *cp);
@@ -91,6 +93,7 @@ int unfactor_cmd(char *cp);
 int fraction_cmd(char *cp);
 int quit_cmd(char *cp);
 int read_cmd(char *cp);
+int read_file(char *cp);
 int read_sub(FILE *fp);
 int edit_cmd(char *cp);
 int save_cmd(char *cp);
@@ -147,6 +150,7 @@ int shell_out(char *cp);
 char *parse_var2(long *vp, char *cp);
 int display_command(int i);
 int read_examples(char **cpp);
+void underline_title(int count);
 int help_cmd(char *cp);
 /* integrate.c */
 void make_powers(token_type *equation, int *np, long v);
@@ -161,6 +165,7 @@ void default_color(void);
 int list1_sub(int n, int export_flag);
 int list_sub(int n);
 void list_debug(int level, token_type *p1, int n1, token_type *p2, int n2);
+char *var_name(long v);
 int list_var(long v, int lang_code);
 int list_proc(token_type *p1, int n, int export_flag);
 char *list_equation(int n, int export_flag);
@@ -172,7 +177,8 @@ int list_c_equation(int en, enum language_list language, int int_flag);
 int list_code(token_type *equation, int *np, enum language_list language, int int_flag);
 int flist_equation(int n);
 /* main.c */
-void usage(void);
+void display_startup_message(FILE *fp);
+void usage(FILE *fp);
 int main(int argc, char **argv);
 void main_io_loop(void);
 int set_signals(void);
@@ -192,7 +198,7 @@ void handle_negate(token_type *equation, int *np);
 void give_priority(token_type *equation, int *np);
 char *parse_section(token_type *equation, int *np, char *cp, int allow_space);
 char *parse_equation(int n, char *cp);
-char *parse_expr(token_type *equation, int *np, char *cp);
+char *parse_expr(token_type *equation, int *np, char *cp, int allow_space);
 char *parse_var(long *vp, char *cp);
 void remove_trailing_spaces(char *cp);
 void set_error_level(char *cp);
@@ -205,6 +211,7 @@ int poly_factor(token_type *equation, int *np, int do_repeat);
 int remove_factors(void);
 int poly_gcd(token_type *larger, int llen, token_type *smaller, int slen, long v);
 int poly2_gcd(token_type *larger, int llen, token_type *smaller, int slen, long v);
+int is_integer_var(long v);
 int is_integer_expr(token_type *p1, int n);
 int mod_simp(token_type *equation, int *np);
 int poly_gcd_simp(token_type *equation, int *np);
@@ -219,7 +226,7 @@ int find_greatest_power(token_type *p1, int n1, long *vp1, double *pp1, int *tp1
 void organize(token_type *equation, int *np);
 void elim_loop(token_type *equation, int *np);
 void simp_ssub(token_type *equation, int *np, long v, double d, int power_flag, int times_flag, int fc_level);
-void simp_sub(int n);
+void simp_equation(int n);
 void simp_side(token_type *equation, int *np);
 void simps_side(token_type *equation, int *np, int zsolve);
 void simpv_side(token_type *equation, int *np, long v);
