@@ -46,7 +46,7 @@ organize(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
-	if (*np <= 0 || (*np & 1) != 1) {
+	if (*np <= 0 || (*np & 1) == 0) {
 		printf("Bad expression size = %d.\n", *np);
 		error_bug("Internal error: organize() called with bad expression size.");
 	}
@@ -56,7 +56,7 @@ int		*np;		/* pointer to length of equation side */
 	org_recurse(equation, np, 0, 1, NULL);
 }
 
-static void
+static inline void
 org_up_level(bp, ep, level, invert)
 token_type	*bp, *ep;
 int		level, invert;
@@ -227,7 +227,7 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
  * No factoring is done.
  */
 void
-simp_sub(n)
+simp_equation(n)
 int	n;	/* equation space number to simplify */
 {
 	if (n_lhs[n] <= 0)
@@ -247,7 +247,7 @@ simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
-	simp_ssub(equation, np, 0L, 1.0, true, true, 6 /* was 2 */);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 6);
 }
 
 /*
@@ -262,7 +262,7 @@ int		zsolve;		/* true for solving for zero */
 	elim_loop(equation, np);
 	simp_constant_power(equation, np);
 	do {
-		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 6 /* was 2 */);
+		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 6);
 	} while (super_factor(equation, np, 0));
 }
 
@@ -277,7 +277,7 @@ long		v;		/* variable to factor, 0 for all variables */
 {
 	if (*np == 0)
 		return;
-	simp_ssub(equation, np, v, 0.0, v == 0, true, 6 /* was 2 */);
+	simp_ssub(equation, np, v, 0.0, v == 0, true, 6);
 }
 
 /*
@@ -490,7 +490,7 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 				simp2_divide(equation, np, va[i].v, fc_level);
 		}
 		simp2_divide(equation, np, 0L, fc_level);
-		/* factor all subexpressions in order of most frequently occurring variables */
+		/* factor all sub-expressions in order of most frequently occurring variables */
 		for (i = 0; i < vc; i++) {
 			while (factor_plus(equation, np, va[i].v, 0.0)) {
 				simp2_divide(equation, np, 0L, fc_level);
@@ -529,13 +529,20 @@ int		*np;		/* pointer to length of equation side */
 
 	make_fractions(equation, np);
 	uf_tsimp(equation, np);
+#if	0	/* Not really needed, but makes it end up the same as the simplify command. */
+	make_fractions(equation, np);
+	uf_power(equation, np);
+	integer_root_simp(equation, np);
+	simpb_side(equation, np, true, true, 3);
+#endif
 	poly_factor(equation, np, true);
 	simpb_side(equation, np, true, false, 2);
+	simpb_side(equation, np, true, false, 2);	/* Added for thoroughness, making sure everything is uf_power()ed. */
 	fractions_and_group(equation, np);
 }
 
 /*
- * This is the slow and thorough simplify.
+ * This is the slow and thorough simplify of the simplify command.
  * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
  * then does generalized polynomial simplifications.
  *
@@ -563,7 +570,7 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	uf_times(equation, np);
 	simp_loop(equation, np);
 
-/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
+/* Here is the only place in Mathomatic that we do complete modulus (%) simplification: */
 	uf_pplus(equation, np);
 	uf_repeat(equation, np);
 	do {
@@ -597,7 +604,7 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 6 /* was 2 */);
+				simp_ssub(equation, np, 0L, 1.0, false, true, 6);
 			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
@@ -615,7 +622,11 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 		/* an error occurred, restore the original expression */
 		*np = n_tlhs;
 		blt(equation, tlhs, n_tlhs * sizeof(token_type));
-		debug_string(1, "Simplify not expanding fully, due to oversized expression.");
+		if (i == 14) {
+			debug_string(1, "Simplify not expanding fully, due to oversized expression.");
+		} else {
+			debug_string(0, "Simplify not expanding fully, due to some error.");
+		}
 		partial_flag = true;	/* expand less */
 		uf_tsimp(equation, np);
 	} else {
@@ -657,7 +668,7 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	}
 /*	simpb_side(equation, np, true, true, 1); */
 	simp_constant_power(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 5 /* was 1 */);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 5);
 	unsimp_power(equation, np);
 	make_fractions(equation, np);
 	factor_power(equation, np);
@@ -1102,12 +1113,13 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 		if (k2 == 0) {
 			warning(_("Modulo 0 encountered, might be considered undefined."));
 		}
-#if	false	/* If fmod() always works.  It doesn't. */
-		*k1p = fmod(*k1p, k2);
-#else
-		/* Another way to get the remainder of division: */
-		*k1p = modf(*k1p / k2, &d) * k2;
-#endif
+		if (fmod(*k1p, 1.0) == 0 && fmod(k2, 1.0) == 0) {
+			*k1p = fmod(*k1p, k2);
+		} else {
+			/* fmod() doesn't work well with fractional amounts. */
+			/* Another way to get the remainder of division: */
+			*k1p = modf(*k1p / k2, &d) * k2;
+		}
 		if (modulus_mode && *k1p < 0.0) {
 			*k1p += fabs(k2);	/* make positive */
 		}
@@ -1141,17 +1153,32 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 				}
 			}
 		}
+#if	1
+		if (errno == ERANGE) {	/* preserve overflowed powers rather than aborting with an error message */
+			domain_check = false;
+			return false;
+		}
+#endif
 		check_err();
 		if (domain_check)
 			*k1p = d;
 		break;
 	case FACTORIAL:
-#if	true	/* set this to false if lgamma() doesn't exist */
+#if	_XOPEN_SOURCE >= 600 || _ISOC99_SOURCE || __USE_ISOC99
+		/* Use true gamma for factorial if available, it is the nicest gamma() function. */
+		d = tgamma(*k1p + 1.0);
+		if (errno) {	/* don't evaluate if overflow */
+			return false;
+		}
+#else
+#if	1	/* Set this to 0 if lgamma() (logarithm of gamma) doesn't exist. */
+		/* Not threadsafe, but neither is most of Mathomatic with its global static data areas. */
 		d = exp(lgamma(*k1p + 1.0)) * signgam;
 		if (errno) {	/* don't evaluate if overflow */
 			return false;
 		}
 #else
+		/* calculate integer factorial if gamma() functions don't exist */
 		if (*k1p > 170.0 || *k1p < 0.0 || fmod(*k1p, 1.0) != 0.0) {
 			return false;
 		}
@@ -1159,6 +1186,7 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 		for (d1 = 2.0; d1 <= *k1p; d1 += 1.0) {
 			d *= d1;
 		}
+#endif
 #endif
 		*k1p = d;
 		break;
@@ -1446,7 +1474,7 @@ int		*np;		/* pointer to length of equation side */
 				}
 			}
 			if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
-/* Move any constant to the beginning of a multiplicative subexpression. */
+/* Move any constant to the beginning of a multiplicative sub-expression. */
 				d = (p1 + 1)->token.constant;
 				for (p2 = p1 - 1; p2 > equation; p2--) {
 					if ((p2 - 1)->level < level)
@@ -1465,11 +1493,10 @@ int		*np;		/* pointer to length of equation side */
 				(p2 + 1)->token.operatr = TIMES;
 				if (p2 > equation) {
 					p1 = p2 - 1;
-					continue;
 				} else {
 					p1 = equation + 1;
-					continue;
 				}
+				continue;
 			}
 			break;
 		case DIVIDE:
@@ -1512,6 +1539,29 @@ int		*np;		/* pointer to length of equation side */
 				modified = true;
 				continue;
 			}
+			if (p1->token.operatr == MODULUS) {
+				if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
+					d = fabs((p1 + 1)->token.constant);
+					if (d > epsilon && fmod(1.0 / d, 1.0) == 0.0) {
+						for (p2 = p1 - 1; p2 > equation; p2--) {
+							if ((p2 - 1)->level < level)
+								break;
+						}
+						if (is_integer_expr(p2, p1 - p2)) {
+							blt(p2, p1 + 1, (char *) ep - (char *) (p1 + 1));
+							*np -= (p1 + 1) - p2;
+							p2->token.constant = 0.0;
+							if (p2 > equation) {
+								p1 = p2 - 1;
+							} else {
+								p1 = equation + 1;
+							}
+							modified = true;
+							continue;
+						}
+					}
+				}
+			}
 			break;
 		case POWER:
 			if (p2->level == level && p2->kind == CONSTANT) {
@@ -1575,7 +1625,12 @@ int		*diff_signp;	/* different sign flag pointer */
 {
 	int	l1, l2;
 
-	/* First, find the proper ground levels of parentheses for the two sub-expressions. */
+	if (((n1 > n2) ? ((n1 + 1) / (n2 + 1)) : ((n2 + 1) / (n1 + 1))) > 3) {
+		/* expressions are grossly different in size, no need to compare, they are different */
+		*diff_signp = false;
+		return false;
+	}
+	/* Find the proper ground levels of parentheses for the two sub-expressions: */
 	l1 = min_level(p1, n1);
 	l2 = min_level(p2, n2);
 	return compare_recurse(p1, n1, l1, p2, n2, l2, diff_signp);
@@ -1591,6 +1646,8 @@ int		n1, l1;
 token_type	*p2;
 int		n2, l2, *diff_signp;
 {
+#define	compare_epsilon	epsilon
+
 	token_type	*pv1, *ep1, *ep2;
 	int		i, j;
 	int		len;
@@ -1598,10 +1655,9 @@ int		n2, l2, *diff_signp;
 	int		oc2;				/* operand count 2 */
 	token_type	*opa2[MAX_COMPARE_TERMS];	/* operand pointer array 2 */
 	char		used[MAX_COMPARE_TERMS];	/* operand used flag array 2 */
-	int		last_op1, op1, op2;
-	int		diff_op;
+	int		last_op1, op1 = 0, op2 = 0;
+	int		diff_op = false;
 	double		d1, c1, c2;
-	double		compare_epsilon = epsilon;
 
 	*diff_signp = false;
 	if (n1 == 1 && n2 == 1) {
@@ -1635,27 +1691,30 @@ int		n2, l2, *diff_signp;
 			}
 			break;
 		case OPERATOR:
+			error_bug("Programming error in call to compare_recurse().");
 			break;
 		}
 		return false;
 	}
+#if	0
+	if ((n1 & 1) != 1 || (n2 & 1) != 1) {
+		error_bug("Programming error in call to compare_recurse().");
+	}
+#endif
 	ep1 = &p1[n1];
 	ep2 = &p2[n2];
-	op1 = 0;
 	for (pv1 = p1 + 1; pv1 < ep1; pv1 += 2) {
 		if (pv1->level == l1) {
 			op1 = pv1->token.operatr;
 			break;
 		}
 	}
-	op2 = 0;
 	for (pv1 = p2 + 1; pv1 < ep2; pv1 += 2) {
 		if (pv1->level == l2) {
 			op2 = pv1->token.operatr;
 			break;
 		}
 	}
-	diff_op = false;
 	if (op2 == 0) {
 		if (op1 != TIMES && op1 != DIVIDE)
 			return false;
@@ -1832,6 +1891,8 @@ int		*np;
 
 	for (i = 1; i < *np; i += 2) {
 		level = equation[i].level;
+#if	1	/* This code makes complex number calculations give different (maybe wrong) results. */
+		/* It converts x/i to x*-1*i.  Might be better to do this only when symb_flag is true. */
 		if (equation[i].token.operatr == DIVIDE) {
 			if (equation[i+1].level == level
 			    && equation[i+1].kind == VARIABLE
@@ -1855,6 +1916,7 @@ int		*np;
 				modified = true;
 			}
 		}
+#endif
 		if (equation[i].token.operatr == POWER
 		    && equation[i+1].level == level
 		    && equation[i+1].kind == CONSTANT) {
@@ -2094,7 +2156,7 @@ int		i1, i2;
 }
 
 /*
- * Recursively check and possibly reorder a "level" subexpression
+ * Recursively check and possibly reorder a "level" sub-expression
  * which starts at "loc" in an equation side.
  */
 static int
@@ -2106,13 +2168,13 @@ int		*np, loc, level;
 	int	op = 0;
 	int	modified = false;
 
-/* check passed subexpression for validity: */
+/* check passed sub-expression for validity: */
 	if ((loc & 1) != 0) {
 		goto corrupt;
 	}
 	for (i = loc; i < *np;) {
 		if (equation[i].level < level) {
-			if (equation[i].kind != OPERATOR) {
+			if (equation[i].kind != OPERATOR || equation[i].level < 1) {
 				goto corrupt;
 			}
 			break;
@@ -2124,11 +2186,8 @@ int		*np, loc, level;
 				;
 			continue;
 		} else {
-			if (((i & 1) == 1) != (equation[i].kind == OPERATOR)) {
-				goto corrupt;
-			}
 			if (equation[i].kind == OPERATOR) {
-				if (!equation[i].token.operatr) {
+				if ((i & 1) == 0 || equation[i].token.operatr == 0) {
 					goto corrupt;
 				}
 				if (op) {
@@ -2148,14 +2207,17 @@ int		*np, loc, level;
 				} else {
 					op = equation[i].token.operatr;
 				}
+			} else {
+				if ((i & 1) != 0)
+					goto corrupt;
 			}
 		}
 		i++;
 	}
-	if ((i & 1) != 1) {	/* "i" should be at the end of the current subexpression */
+	if ((i & 1) == 0) {	/* "i" should be at the end of the current sub-expression */
 		goto corrupt;
 	}
-/* reorder subexpression if needed: */
+/* reorder sub-expression if needed: */
 	switch (op) {
 	case PLUS:
 	case MINUS:
